<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Http\Classes\MyAuth;
use App\Models\compaign;
use Exception;
use Illuminate\Support\Str;
use App\Models\tag;
use App\Models\company;
use App\Models\compaign_receivable;
use Illuminate\Support\Facades\Validator;
use App\Jobs\SendCompiagn;
use Illuminate\Support\Facades\Bus;
use Throwable;
use App\Http\Traits\SystemMail;
use Illuminate\Bus\Batch;
use App\Models\mailing_list;
use App\Models\mailing_file;
use App\Models\template;

use App\Models\email;
class CompaignController extends Controller
{
    use SystemMail;
    public function profile($id)
    {
        $compaign =  compaign::where("externalid", $id)->with("get_email:id,email")->with("get_customers:id,name,email,externalid")->with("get_mailing_list:id,name,email,externalid")->with("get_creater")->first();
        if (!$compaign) {
            return ['message' => 'Compaign Not Found'];
        }
        return view("compaign.profile", compact("compaign"));
    }
    public function public_profile($id)
    {
        $compaign =  compaign::where("externalid", $id)->first();
        if (!$compaign)
        {
            return ['message' => 'Compaign Not Found'];
        }
        return view("compaign.open_profile", compact("compaign"));
    }

    public function recipient_sharing_detail($pid)
    {
       $detail =  compaign_receivable::where("id",$pid)->with("get_compaign:id,externalid,title,short_description,email_id,company_name","get_compaign.get_email")->with("get_messages")->with("compaign_receivable:id,name,email,externalid")->first();
        if(!$detail)
        {
            return ['message' => 'Compaign Not Found'];
        }
        return view("compaign.recipient_sharing_detail", compact("detail"));
    }
    public function share(Request $req, $id)
    {
        // $req->session()->pull("cms_batch_id");
        if ($req->method() == "POST")
        {
            $query = compaign::query();
            if ($req->type == "all") {
                $query->with("get_customers:id,name,email,externalid")->with("get_mailing_list:id,name,email,externalid");
            } else  if ($req->type == "draft") {
                $query->with(["get_customers" => function ($q) {
                    $q->wherePivot("sending_status", "draft")
                        ->select("companies.id", "companies.name", "companies.email", "companies.externalid");
                }])->with(["get_mailing_list" => function ($q) {
                    $q->wherePivot("sending_status", "draft")
                        ->select("mailing_lists.id", "mailing_lists.name", "mailing_lists.email", "mailing_lists.externalid");
                }]);
            }
            $compaign = $query->where("externalid", $req->compaign_id)->first();
            if (!$compaign) {
                return ['message' => 'Compaign Not Found'];
            }
            if(sizeof($compaign->get_customers ?? []) <= 0 && sizeof($compaign->get_mailing_list ?? []) <= 0)
            {
                return redirect()->back()->with(['form_submit_flag' => 'false', 'form_submit_msg' => 'No recipients found']);
            }

            $jobs = array();
            $sec = 1;
            $data = array();
            $data['subject'] = $compaign->title;
            $data['description'] = $compaign->content;
            foreach ($compaign->get_customers as $cust)
            {
                $jobs[] = new SendCompiagn($cust, $cust->pivot->id, $data, "customer",$compaign);
            }

            foreach ($compaign->get_mailing_list as $cust)
            {
                $jobs[] = new SendCompiagn($cust, $cust->pivot->id, $data, "mailing_list",$compaign);
            }

            $total_jobs = sizeof($jobs);
            SendCompiagn::$counter = 0;
            SendCompiagn::$total_excluded = array();
            $batch = Bus::batch($jobs)->then(function (Batch $batch) use ($total_jobs, $compaign) {
                $this->mail_body['description'] = " The Compaign (" . $compaign->title . ") Sent to " . SendCompiagn::$counter . " out of " . $total_jobs . " receipent(s). ";

                if (sizeof(SendCompiagn::$total_excluded) > 0) {
                    $this->mail_body['description'] = $this->mail_body['description'] . " The Compaign (" . $compaign->title . ") was not sent to following recipients:<br>" . SendCompiagn::$total_excluded;
                }
                $users = array();
                $this->mail_subject = "Compaign Sent";
                $this->mail_link = "compaign/profile/" . $compaign->externalid;
                $users = $this->get_mail_members($this->mail_subject, $this->mail_body, $this->mail_link, ['manager_12', 'super_admin_11'], [], $users);
                $this->send_to_members("", "", "", $users, "false", "2");
                compaign::where("id",$compaign->id)->update(['status' => 'sent']);
            })->catch(function (Batch $batch, Throwable $e) {
                // First batch job failure detected...
            })->finally(function (Batch $batch) {
                // The batch has finished executing...
            })->allowFailures(false)
                ->dispatch();
            $req->session()->put("cms_batch_id", $batch->id);
            $req->session()->put("cms_batch_desc","Sending Compaign");
            return redirect()->back()->with(['form_submit_flag' => 'true', 'form_submit_msg' => 'Recipients Added in Queue. You will be notified as the process completed']);

        }
        $compaign =  compaign::where("externalid", $id)->with("get_customers:id,name,email,externalid")->with("get_mailing_list:id,name,email,externalid")->with("get_creater")->first();
        if (!$compaign) {
            return ['message' => 'Compaign Not Found'];
        }
        return view("compaign.share", compact("compaign"));
    }

    public function editor(Request $req, $id)
    {
        $compaign =  compaign::where("externalid", $id)->first();
        if (!$compaign)
        {
            return ['message' => 'Compaign Not Found'];
        }
        if ($req->method() == "POST")
        {
            return $req->all();
        }
        $templates = template::select("title","description","content")->get();
        return view("compaign.editor", compact("compaign","templates"));
    }

    public function recipients(Request $req, $id, $type = "")
    {
        $filters = array();
        if ($type == "customer" || $type == "") {
            $type = "customer";
            $compaign =  compaign::with("get_customers:id,name,email")
                ->where("externalid", $id)->first();

            if (!$compaign) {
                return ['message' => 'Compaign Not Found'];
            }
            $companies = array();
            $query = company::query();

            if ($req->email && $req->email != "") {
                $filters['email'] = $req->email;
                $query = $query->where('email', $req->email);
            }
            if ($req->name && $req->name != "") {
                $filters['name'] = $req->name;

                $query = $query->where('name', 'LIKE', '%' . $req->name . '%');
            }
            if ($req->tags && sizeof($req->tags) >= 0) {
                $filters['tags'] = $req->tags;
                foreach ($req->tags as $t) {
                    $query = $query->whereHas("get_tags", function ($q) use ($t) {
                        $q->where("tags.id", $t);
                    });
                }
            }
            $tags = tag::limit(10)->get();
            $recipients =  $query->select("id", "name", "email", "phone", "externalid")
                ->with(["get_compaigns" => function ($q) use ($id) {
                    $q->where("compaigns.externalid", $id)->select("compaigns.id", "compaigns.title");
                }])->withCount(["get_compaigns" => function ($q) use ($id) {
                    $q->where("compaigns.externalid", $id)->select("compaigns.id", "compaigns.title");
                }])->orderBy("get_compaigns_count", "desc")->paginate(20);
                return view("compaign.recipients", compact("tags", "filters", "recipients", "compaign","type"));

        }
        else if($type == "mailing_list")
        {
            $compaign =  compaign::with("get_mailing_list:id,name,email")
            ->where("externalid", $id)->first();

        if (!$compaign) {
            return ['message' => 'Compaign Not Found'];
        }
        $query = mailing_list::query();
        if($req->email && $req->email!="")
        {
            $filters['email'] = $req->email;
            $query->where("email","like","%".$req->email."%");
        }
        if($req->name && $req->name!="")
        {
            $filters['name'] = $req->name;

            $query->where("name","like","%".$req->name."%");
        }
        if($req->group && $req->group!="")
        {
            $filters['group'] = $req->group;

            $query->where("mailing_file_id",$req->group);
        }
        $groups = mailing_file::all();
        $recipients = $query
        ->with("get_group")->with(["get_compaigns" => function ($q) use ($id) {
            $q->where("compaigns.externalid", $id)->select("compaigns.id", "compaigns.title");
        }])->withCount(["get_compaigns" => function ($q) use ($id) {
            $q->where("compaigns.externalid", $id);
        }])->orderBy("get_compaigns_count", "desc")->paginate(20);
        }

        return view("compaign.recipients", compact("groups", "filters", "recipients", "compaign","type"));
    }

    public function delete(Request $req)
    {
        if(compaign::where("externalid", $req->compaign_id)->delete())
        {
            return redirect()->back()->with(['form_submit_flag' => 'true', 'form_submit_msg' => 'Compaign Deleted Successfully']);
        }
        else
        {
            return redirect()->back()->with(['form_submit_flag' => 'false', 'form_submit_msg' => 'Compaign was not deleted or not found']);
        }
    }


    public function save_recipients(Request $req)
    {
        $arr = array();
        try {
            if($req->recipient_type == "customer" || $req->recipient_type == ""){
            if ($req->action == "add") {
                $old = compaign_receivable::where("compaign_id", $req->compaign_id)->where("compaign_receivable_type", "App\Models\company")->select("id", "compaign_receivable_id")->get();
                $exist = collect($old)->pluck("compaign_receivable_id")->toArray();
                // return $exist;
                foreach ($req->recipients  as $rec) {
                    if (in_array($rec, $exist)) {
                        continue;
                    }
                    $arr[] = array(
                        'compaign_id' => $req->compaign_id, "compaign_receivable_id" => $rec,
                        "compaign_receivable_type" => "App\Models\company", "sending_status" => 'draft', "created_at" => date("Y-m-d H:i:s"),
                        "updated_at" => date("Y-m-d H:i:s")
                    );
                }

                if (sizeof($arr) > 0) {
                    if (compaign_receivable::insert($arr)) {
                        return redirect()->back()->with(['form_submit_flag' => 'true', 'form_submit_msg' => 'Recipients Added Successfully']);
                    } else {
                        return redirect()->back()->with(['form_submit_flag' => 'false', 'form_submit_msg' => 'Recipients were not Added. Try Again']);
                    }
                } else {
                    return redirect()->back()->with(['form_submit_flag' => 'false', 'form_submit_msg' => 'No receipient was found']);
                }
            } else if ($req->action == "remove") {
                if (compaign_receivable::whereIn("compaign_receivable_id", $req->recipients)->where("compaign_id", $req->compaign_id)
                    ->where("sending_status", "draft")->where("compaign_receivable_type", "App\Models\company")->delete()
                ) {
                    return redirect()->back()->with(['form_submit_flag' => 'true', 'form_submit_msg' => 'Recipients removed Successfully']);
                } else {
                    return redirect()->back()->with(['form_submit_flag' => 'false', 'form_submit_msg' => 'Recipients were not Deleted. Try Again']);
                }
            }
        }
            else if($req->recipient_type == "mailing_list")
            {
                if ($req->action == "add") {
                    $old = compaign_receivable::where("compaign_id", $req->compaign_id)->where("compaign_receivable_type", "App\Models\mailing_list")->select("id", "compaign_receivable_id")->get();
                    $exist = collect($old)->pluck("compaign_receivable_id")->toArray();
                    // return $exist;
                    foreach ($req->recipients  as $rec) {
                        if (in_array($rec, $exist)) {
                            continue;
                        }
                        $arr[] = array(
                            'compaign_id' => $req->compaign_id, "compaign_receivable_id" => $rec,
                            "compaign_receivable_type" => "App\Models\mailing_list", "sending_status" => 'draft', "created_at" => date("Y-m-d H:i:s"),
                            "updated_at" => date("Y-m-d H:i:s")
                        );
                    }

                    if (sizeof($arr) > 0) {
                        if (compaign_receivable::insert($arr)) {
                            return redirect()->back()->with(['form_submit_flag' => 'true', 'form_submit_msg' => 'Recipients Added Successfully']);
                        } else {
                            return redirect()->back()->with(['form_submit_flag' => 'false', 'form_submit_msg' => 'Recipients were not Added. Try Again']);
                        }
                    } else {
                        return redirect()->back()->with(['form_submit_flag' => 'false', 'form_submit_msg' => 'No receipient was found']);
                    }
                } else if ($req->action == "remove") {
                    if (compaign_receivable::whereIn("compaign_receivable_id", $req->recipients)->where("compaign_id", $req->compaign_id)
                        ->where("sending_status", "draft")->where("compaign_receivable_type", "App\Models\mailing_list")->delete()
                    ) {
                        return redirect()->back()->with(['form_submit_flag' => 'true', 'form_submit_msg' => 'Recipients removed Successfully']);
                    } else {
                        return redirect()->back()->with(['form_submit_flag' => 'false', 'form_submit_msg' => 'Recipients were not Deleted. Try Again']);
                    }
                }
            }
        } catch (Exception $ex) {
            return redirect()->back()->with(['form_submit_flag' => 'false', 'form_submit_msg' => 'An Exception Occured. ' . $ex->getMessage() . " Try Again"]);
        }
    }

    public function view_all(Request $req)
    {
        $filters = array();
        $query = compaign::query();
        if($req->input("title") && $req->input("title")!="")
        {
            $filters['title'] = $req->title;
            $query->where("title","like","%".$req->title."%");
        }
        if($req->input("id") && $req->input("id")!="")
        {
            $filters['id'] = $req->id;

            $query->where("externalid",$req->id);
        }
        $compaigns =  $query->paginate(config("app.paginate"));
        return view("compaign.view_all", compact("compaigns","filters"));
    }

    public function create(Request $req)
    {
        if ($req->method() == "POST") {
            $req->validate(
                [
                    'subject' => 'required|min:3|max:100',
                    'short_description' => 'nullable|max:150',
                    'email' => 'required|exists:emails,id',
                    'company_name' => 'required|min:3|max:200',
                ]
            );
            try {
                $com = new compaign();
                $com->title = $req->subject;
                $com->company_name = $req->company_name;
                $com->email_id = $req->email;
                $com->externalid = uniqid();
                $com->short_description = $req->short_description;
                $com->status = "draft";
                $com->created_by = MyAuth::auth('member')->id;
                if ($com->save()) {
                    return redirect()->route("compaign.editor",$com->externalid)->with(['form_submit_flag' => 'true', 'form_submit_msg' => 'Compaign Created Successfully. Now You can edit Content']);
                } else {
                    return redirect()->back()->with(['form_submit_flag' => 'true', 'form_submit_msg' => 'Compaign was not Created. Try Again']);
                }
            } catch (Exception $e) {
                return redirect()->back()->with(['form_submit_flag' => 'false', 'form_submit_msg' => 'An Exception Occured' . $e->getMessage()]);
            }
        }
        $emails = email::select("email","id")->get();
        return view("compaign.create", compact("emails"));
    }

    public function edit(Request $req)
    {
        if ($req->method() == "POST") {
            $req->validate(
                [
                    'subject' => 'required|min:3|max:100',
                    'short_description' => 'nullable|max:150',
                    'email' => 'required|exists:emails,id',
                    'company_name' => 'required|min:3|max:200',
                ]
            );
            try {
                $compaign = compaign::where("externalid",$req->compaign_id)->first();
                if(!$compaign)
                {
                    return redirect()->back()->with(['form_submit_flag' => 'false', 'form_submit_msg' => 'Compaign Not Found.']);
                }
                $compaign->title = $req->subject;
                $compaign->company_name = $req->company_name;
                $compaign->email_id = $req->email;
                $compaign->short_description = $req->short_description;
                if ($compaign->save())
                {
                    if($compaign->wasChanged())
                    {
                        return redirect()->back()->with(['form_submit_flag' => 'true', 'form_submit_msg' => 'Compaign Updated Successfully.']);
                    }
                    else
                    {
                        return redirect()->back()->with(['form_submit_flag' => 'false', 'form_submit_msg' => 'Nothing was Changed.']);
                    }
                }
                else
                {
                    return redirect()->back()->with(['form_submit_flag' => 'false', 'form_submit_msg' => 'Compaign was not Updated. Try Again']);
                }
            }
            catch (Exception $e)
            {
                return redirect()->back()->with(['form_submit_flag' => 'false', 'form_submit_msg' => 'An Exception Occured' . $e->getMessage()]);
            }
        }
        $emails = email::select("email","id")->get();
        $compaign = compaign::where("externalid",$req->id)->with("get_email")->first();
        if(!$compaign)
        {
            return ['message' => 'Compaign Not Found'];
        }
        return view("compaign.edit", compact("emails","compaign"));
    }

    public function update_content(Request $req)
    {
        try {
            $rules['compaign_id'] =  'required|exists:compaigns,externalid';
            $rules['content'] =  'required';
            $validator = Validator::make($req->all(), $rules);
            if ($validator->fails()) {
                return response()->json(['status' => 'fail', 'time' => date("Y-m-d H:i:s"), 'message' => 'Validation Errors. Compaign Might not found or some error in content', 'errors' => $validator->errors()]);
            }
            $compaign =  compaign::where("externalid", $req->compaign_id)->first();
            $compaign->content = $req->content;
            if ($compaign->save()) {
                if ($compaign->wasChanged()) {
                    return response()->json(['status' => 'success', 'time' => date("Y-m-d H:i:s"), 'message' => 'Content Updated']);
                } else {
                    return response()->json(['status' => 'fail', 'time' => date("Y-m-d H:i:s"), 'message' => 'No changes were made']);
                }
            } else {
                return response()->json(['status' => 'fail', 'time' => date("Y-m-d H:i:s"), 'message' => 'Content was not Updated']);
            }
        } catch (Exception $e) {
            return response()->json(['status' => 'fail', 'message' => "Exception Occured" . $e->getMessage()]);
        }
    }
}
