<?php

namespace App\Http\Controllers;

use App\Models\department;
use Illuminate\Http\Request;
use Exception;
use App\Models\member;
use App\Http\Traits\SystemMail;
class DepartmentController extends Controller
{
    use SystemMail;
    public function add_department(Request $req)
    {

        $supervisors = member::select("id","name")->whereRelation("get_role","role_code","=","supervisor_13")->get();

        if($req->method() == "POST")
        {
            $req->validate([
                'name'=>'required|min:3|max:60',
                'supervisor'=>'nullable|exists:members,id',
                'time' =>'required|integer|min:1|max:50000',
                'description'=>'nullable|max:2999',
            ]);

            try
            {
                $department = new department();
                $department->externalid = "department_".time();
                $department->name = $req->name;

                $department->description = $req->description;
                $department->ticket_time = $req->time;
                $department->supervisor_id = $req->supervisor;
                $department->added_by = session("cms_member_id");
                $desc = "";
                if($department->save())
                {
                    $supervisor= member::where("id",$req->supervisor)->first();
                    if($supervisor)
                    {
                        $supervisor->department_id = $department->id;
                        $supervisor->save();
                        $desc = "A new department ( ".$department->name." ) has been added under the supervison of ".$supervisor->name." by ".session("cms_member_name")." (".session("cms_member_role_name").")";
                    }
                    else
                    {
                        $desc = "A new department ( ".$department->name." ) has been added" ." by ".session("cms_member_name")." (".session("cms_member_role_name").")";
                    }


                        $users = array();
                        $this->mail_subject ="New Department Added";
                        $this->mail_body['description'] = $desc;
                        $this->mail_link = "department_profile/".$department->externalid;
                        $users = $this->get_mail_members($this->mail_subject, $this->mail_body, $this->mail_link, ['manager_12','super_admin_11'],[],$users);
                        if($supervisor)
                        {
                            $this->mail_body['description']  = "A new department ( ".$department->name." ) has been added under your supervison "." by ".session("cms_member_name")." (".session("cms_member_role_name").")";

                            $users = $this->get_specific_member($this->mail_subject, $this->mail_body, $this->mail_link, $supervisor->id,$users);
                        }
                        dispatch(
                            function () use ($users)
                            {
                                $this->send_to_members("","","",$users,"false","2");
                            }
                        )->delay(now()->addSeconds(config("app.queue_time")));

                    return redirect()->back()->with(['form_submit_flag' => 'true', 'form_submit_msg' => 'Department Added Successfully']);
                }
            }
            catch(Exception $e)
            {
                return redirect()->back()->with(['form_submit_flag' => 'false', 'form_submit_msg' => 'An Exception Occured'.$e->getMessage()]);
            }
        }
        return view("department.add_department",compact("supervisors"));
    }

    public function delete_department(Request $req)
    {
        $department = department::whereDoesntHave("get_members")->whereDoesntHave("get_tickets")->where("id",$req->id)->first();
        if($department->delete())
        {
            return redirect()->back()->with(['form_submit_flag' => 'true', 'form_submit_msg' => "Department has deleted."]);
        }
        else
        {
            return redirect()->back()->with(['form_submit_flag' => 'false', 'form_submit_msg' => "Department was not deleted. Try Again."]);
        }
    }

    public function edit_department(Request $req,$id)
    {
        // return $req->path();
        $dep = department::where("externalid", $id)->first();
        if(!$dep)
        {
            return ['message' => 'Department Not Found'];
        }
        $supervisors = member::select("id","name")->whereRelation("get_role","role_code","=","supervisor_13")->get();

        if($req->method() == "POST")
        {
            $req->validate([
                'name'=>'required|min:3|max:60',
                'supervisor'=>'nullable|exists:members,id',
                'time' =>'required|integer|min:1|max:50000',

                'description'=>'nullable|max:2999',
            ]);

            try
            {
                $sup_temp = $dep->supervisor_id;
                $dep->name = $req->name;
                $dep->ticket_time = $req->time;

                $dep->description = $req->description;
                $dep->supervisor_id = $req->supervisor;
                $desc = "";
                if($dep->save())
                {
                    if($dep->wasChanged())
                    {
                    $supervisor= member::where("id",$req->supervisor)->first();
                    if($supervisor)
                    {
                        $supervisor->department_id = $dep->id;
                        $supervisor->save();
                        $desc = "The department ( ".$dep->name." ) has been updated that is under the supervison of ".$supervisor->name.""." by ".session("cms_member_name")." (".session("cms_member_role_name").")";
                    }
                    else
                    {
                        $desc = "The department ( ".$dep->name." ) has been updated"." by ".session("cms_member_name")." (".session("cms_member_role_name").")";
                    }

                    if($sup_temp != $dep->supervisor_id)
                    {
                        $super= member::where("id",$sup_temp)->where("department_id",$dep->id)->update(['department_id'=>null]);
                    }

                        $users = array();
                        $this->mail_subject =" Department Updated";
                        $this->mail_body['description'] = $desc;
                        $this->mail_link = "department_profile/".$dep->externalid;
                        $users = $this->get_mail_members($this->mail_subject, $this->mail_body, $this->mail_link, ['manager_12','super_admin_11'],[],$users);
                        if($supervisor)
                        {
                            $this->mail_body['description']  = "The department ( ".$dep->name." ) has been updated that is under your supervison"." by ".session("cms_member_name")." (".session("cms_member_role_name").")";
                            $users = $this->get_specific_member($this->mail_subject, $this->mail_body, $this->mail_link, $supervisor->id,$users);
                        }
                        dispatch(
                            function () use ($users)
                            {
                                $this->send_to_members("","","",$users,"false","2");
                            }
                        )->delay(now()->addSeconds(config("app.queue_time")));

                        return redirect()->back()->with(['form_submit_flag' => 'true', 'form_submit_msg' => 'Department Edited Successfully']);
                    }
                    else
                    {
                        return redirect()->back()->with(['form_submit_flag' => 'false', 'form_submit_msg' => 'Nothing was changed']);

                    }

                }
                else
                {
                    return redirect()->back()->with(['form_submit_flag' => 'false', 'form_submit_msg' => 'Department Was not edited Successfully']);
                }
            }
            catch(Exception $ex)
            {
                return redirect()->back()->with(['form_submit_flag' => 'false', 'form_submit_msg' => "An Exception Occured. ".$ex->getMessage()]);
            }
        }

        return view("department.edit_department",compact("supervisors","dep"));

    }

    public function department_profile($id)
    {
        $department = department::where("externalid", $id)->first();
        if(!$department)
        {
            return redirect()->back()->with(['form_submit_flag' => 'false', 'form_submit_msg' => 'Department Not Found']);
        }
        return view('department.department_profile',compact("department"));

    }

    public function view_departments(Request $req)
    {
        if($req->name && $req->name != "") {
            $departments = department::where('name', 'LIKE', '%' . $req->name . '%')->orderBy("id", "desc")->paginate(config("app.paginate"));
            return view("department.view_departments", compact("departments"));
        }
        $departments = department::with("get_supervisor")->withCount("get_tickets")->withCount("get_members")->orderBy("id","desc")->withCount("get_pending_tickets")->paginate(config("app.paginate"));
        return view("department.view_departments", compact("departments"));

    }

    public function my_departments()
    {
        $departments = department::with("get_supervisor")->withCount("get_tickets")->withCount("get_pending_tickets")->where("supervisor_id",session("cms_member_id"))->paginate(config("app.paginate"));

        return view("department.view_departments", compact("departments"));
    }
}
