<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Models\mailing_list;
use App\Imports\TestingImport;
use Excel;
use App\Models\mailing_file;
use Exception;
use App\Models\compaign;
use App\Models\compaign_receivable;
class MailingListController extends Controller
{
    public function import(Request $req)
    {
        $groups = mailing_file::all();
        $compaign = [];
        if($req->compaign_id && $req->compaign_id!="")
        {
            $compaign = compaign::select("id","title")->where("externalid",$req->compaign_id)->first();
            if(!$compaign)
            {
                return ['message' => 'Compaign Not Found'];
            }
        }
        return view("compaign.mailing_list.import" ,compact("groups","compaign"));
    }

    public function create(Request $req)
    {
        if($req->method() == "POST")
        {
            $req->validate(
                ['name' =>'required|min:3|max:150',
                'email' => 'required|min:2|max:150|unique:mailing_lists,email',
                'list' => 'required|exists:mailing_files,id'
                ]
            );

            // return $req->all();
            $ml = new mailing_list();
            $ml->name = $req->name;
            $ml->email = $req->email;
            $ml->externalid = "list_".time();
            $ml->mailing_file_id = $req->list;
            try{
            if($ml->save())
            {
                return redirect()->back()->with(['form_submit_flag' => 'true', 'form_submit_msg' => "Mailing List Added Successfully."]);
            }
            else
            {
                return redirect()->back()->with(['form_submit_flag' => 'false', 'form_submit_msg' => "Mailing List not added. Try Again"]);
            }
        }
        catch(Exception $e)
        {
            return redirect()->back()->with(['form_submit_flag' => 'false', 'form_submit_msg' => "Exception Occured. ". $e->getMessage()]);
        }


        }
        $lists = mailing_file::all();
        return view("compaign.mailing_list.create",compact("lists"));
    }

    public function edit(Request $req,$id)
    {
        if($req->method() == "POST")
        {
            $ml = mailing_list::where("externalid",$req->id)->first();
            if(!$ml)
            {
                return ['message' => 'Mailing list not found'];
            }

            $req->validate(
                ['name' =>'required|min:3|max:150',
                'email' => 'required|min:2|max:150|unique:mailing_lists,email,'.$ml->id,
                'list' => 'required|exists:mailing_files,id'
                ]
            );

            // return $req->all();

            $ml->name = $req->name;
            $ml->email = $req->email;
            $ml->mailing_file_id = $req->list;
            try
            {
                if($ml->save())
                {
                    if($ml->wasChanged())
                    {
                        return redirect()->back()->with(['form_submit_flag' => 'true', 'form_submit_msg' => "Mailing List Updated Successfully."]);
                    }
                    else
                    {
                        return redirect()->back()->with(['form_submit_flag' => 'false', 'form_submit_msg' => "Nothing was changed."]);
                    }
                }
                else
                {
                    return redirect()->back()->with(['form_submit_flag' => 'false', 'form_submit_msg' => "Mailing List not Updated. Try Again"]);
                }
        }
        catch(Exception $e)
        {
            return redirect()->back()->with(['form_submit_flag' => 'false', 'form_submit_msg' => "Exception Occured. ". $e->getMessage()]);
        }


        }
        $lists = mailing_file::all();
        $ml = mailing_list::where("externalid",$req->id)->first();
        if(!$ml)
        {
            return ['message' => 'Mailing list not found'];
        }
        return view("compaign.mailing_list.edit",compact("lists","ml"));
    }



    public function submit_import(Request $req)
    {

        $req->validate(['group'=>'required|exists:mailing_files,id',
        'compaign_id' => 'nullable|exists:compaigns,id'
    ]);

        $path = "";
        $groups = mailing_file::all();
        if ($req->file("file"))
        {
            $file = $req->file("file");
            $path = $req->file("file")->getRealPath();
        }
        $old_users = mailing_list::select("email")->get();
        $old_users_email = collect($old_users)->pluck('email')->toArray();

        $data = Excel::toArray(new TestingImport, $path)[0];
        $data_keys = array_keys($data[0]);
        $required_columns = array("name","email");
        $row_counter=0;
        $message = array();
        foreach($required_columns as $key)
        {
            if (!(array_key_exists($key, $data[0])))
            {
                return redirect()->back()->with(['form_submit_flag' => 'false', 'form_submit_msg' => "Name or Email column is missing in file"]);
            }
        }

        $lists = array();
        $counter=0;
        // return $data;
        $ver = time();

        foreach($data as $imp)
        {
            if(in_array($imp['email'],$old_users_email))
            {
                $message[]  = array("row_counter"=> ++$row_counter,"email"=>($imp['email'] ?? 'N/A'),"status" => "fail", "message"=>"Email Already Exists");
                continue;
            }
            if($imp['name']!="" && $imp['name']!="" )
            {
                $counter++;
                $lists[] = array("name"=>$imp['name'], "email"=>$imp['email'],"externalid"=>"list_".time().$counter,
                "mailing_file_id"=>$req->group,"version"=>$ver,
                "created_at"=>date("Y-m-d H:i:s"),"updated_at"=>date("Y-m-d H:i:s"));
                $old_users_email[] = $imp['email'];
            }
            else
            {
                $message[]  = array("row_counter"=> ++$row_counter, "email"=>($imp['email'] ?? 'N/A'), "status" => "fail", "message"=>"Email or Name is missing");
                continue;
            }
        }
        if(sizeof($lists) > 0)
        {
            if(mailing_list::insert($lists))
            {
                session()->flash('form_submit_flag', "true");
                session()->flash('form_submit_message', $message);

                session()->flash('form_submit_msg',  $counter." out of ".sizeof($data)." list have been imported.");
                if($req->compaign_id && $req->compaign_id!="")
                {
                    $arr = array();
                    $new_data = mailing_list::select("id")->where("version",$ver)->get();
                    foreach($new_data as $l)
                    {
                        $arr[] = array(
                            'compaign_id' => $req->compaign_id, "compaign_receivable_id" => $l->id,
                            "compaign_receivable_type" => "App\Models\mailing_list", "sending_status" => 'draft', "created_at" => date("Y-m-d H:i:s"),
                            "updated_at" => date("Y-m-d H:i:s")
                        );
                    }
                    if (compaign_receivable::insert($arr))
                    {
                        session()->flash('form_submit_msg',  $counter." out of ".sizeof($data)." list have been imported. and also added in compaign");
                    }
                    else
                    {
                        session()->flash('form_submit_msg',  $counter." out of ".sizeof($data)." list have been imported. but not added in compaign");
                    }
                }
                return redirect()->back();
            }
            else
            {
                session()->flash('form_submit_flag', "false");
                session()->flash('form_submit_msg',  "Something went wrong");
                session()->flash('form_submit_message', $message);
                return redirect()->back();
            }
        }
        else
        {
            session()->flash('form_submit_flag', "false");
            session()->flash('form_submit_msg', "No List Imported");
            session()->flash('form_submit_message', $message);
            return redirect()->back();
        }
    }

    public function add_file_name(Request $req)
    {
        $req->validate(['name'=>'required|max:200']);
        try
        {
            $file = new mailing_file();
            $file->name = $req->name;
            if($file->save())
            {
                return redirect()->back()->with(['form_submit_flag' => 'true', 'form_submit_msg' => "New File Name Added"]);
            }
            else
            {
                return redirect()->back()->with(['form_submit_flag' => 'false', 'form_submit_msg' => "New File Name Added"]);
            }
        }
        catch(Exception $e)
        {
            return redirect()->back()->with(['form_submit_flag' => 'false', 'form_submit_msg' => "Exception Occured".$e->getMessage()]);
        }
    }

    public function view(Request $req)
    {
        $params = array();
        $query = mailing_list::query();
        if($req->email && $req->email!="")
        {
            $params['email'] = $req->email;
            $query->where("email","like","%".$req->email."%");
        }
        if($req->name && $req->name!="")
        {
            $params['name'] = $req->name;

            $query->where("name","like","%".$req->name."%");
        }
        if($req->group && $req->group!="")
        {
            $params['group'] = $req->group;

            $query->where("mailing_file_id",$req->group);
        }
        $groups = mailing_file::all();
        $list = $query->with("get_group")->paginate( config("app.paginate") );
        return view("compaign.mailing_list.view", compact("list","groups","params"));
    }
}
