<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Models\role;
use App\Models\member;
use Exception;
use App\Models\department;
use App\Http\Traits\SystemMail;
use Illuminate\Support\Facades\Auth;
class MemberController extends Controller
{
    use SystemMail;

    public function my_profile()
    {
        $member=member::where('id',session('cms_member_id'))->first();
        if($member)
        {
            return view('member.my_profile',compact("member"));
        }
        else{
            return redirect()->back()->with(['form_submit_flag' => 'false', 'form_submit_msg' => 'Member Not Found' ]);
        }
    }

    public function reset_my_password(Request $req)
    {
        $req->validate([
            'new_pass'=>'required',
            'old_pass'=>'required',
        ]);
        $member = member::find(session('cms_member_id'));
        if($member)
        {
           if($member->password==$req->old_pass)
           {
              $member->password=$req->new_pass;
              if($member->save())
              {
                return redirect()->back()->with(['form_submit_flag' => 'true', 'form_submit_msg' => 'Password Reset Successfuly']);
              }
           }
           else{
            return redirect()->back()->with(['form_submit_flag' => 'false', 'form_submit_msg' => 'Old password does not match. Try again.']);
           }
        }
        else{
            return redirect()->back()->with(['form_submit_flag' => 'false', 'form_submit_msg' => 'Account Not Found.']);
        }
    }
    //
    public function member_profile($id="")
    {
        if($id=="")
        {
            $member=member::where('id',session("cms_member_id"))->first();
        }
        else
        {
            $member=member::where('externalid',$id)->first();
        }
        if($member)
        {
            return view('member.member_profile',compact("member"));
        }
        else{
            return redirect()->back()->with(['form_submit_flag' => 'false', 'form_submit_msg' => 'Member Not Found' ]);
        }
    }
    public function reset_password_by_manager(Request $req)
    {
        $req->validate([
            'new_pass'=>'required|min:8|max:16',
            'member_id'=>'required',
        ]);

        $member=member::find($req->member_id);
        if($member)
        {
            $member->password=$req->new_pass;
            if($member->save())
            {
                if($member->wasChanged())
                {
                    $users = array();
                    $this->mail_subject ="Your Password Reset";
                    $this->mail_body['description'] = "Your Password has been reset and new password is: ".$req->new_pass;
                    $this->mail_link = "login";
                    $users = $this->get_specific_member($this->mail_subject, $this->mail_body, $this->mail_link, $req->member_id,$users);

                     dispatch(
                        function () use ($users)
                        {
                            $this->send_to_members("","","",$users,"false","2");
                        }
                    )->delay(now()->addSeconds(2));

                    return redirect()->back()->with(['form_submit_flag' => 'true', 'form_submit_msg' => 'Password Reset Successfuly' ]);
                }
                else
                {
                    return redirect()->back()->with(['form_submit_flag' => 'false', 'form_submit_msg' => 'You have entered old Password' ]);
                }
            }
            else{
                return redirect()->back()->with(['form_submit_flag' => 'false', 'form_submit_msg' => 'Error!Try again' ]);
            }
        }
        else{
            return redirect()->back()->with(['form_submit_flag' => 'false', 'form_submit_msg' => 'Member Not Found' ]);
        }
    }
    public function add_member(Request $req)
    {
        $roles = role::select("id","name","role_code")->where("level",">",2)->get();
        $departments = department::select("id","name","supervisor_id")->with("get_supervisor")->get();
        if(sizeof($departments)<=0)
        {
            return view("lock_pages.lock_page",['message'=>'No Department Exists till now. Kindly Add a Department First']);
        }
        $managers = member::select("id","name")->whereRelation("get_role","role_code","=","manager_12")->get();

        if($req->method() == "POST")
        {
            $req->validate([
                'name'=>'required|min:3|max:60',
                'email'=>'required|email|max:200|unique:members,email',
                'phone'=>'required|min:10|max:15|unique:members,phone',
                'role'=>'required|exists:roles,role_code',
                'department'=>'nullable|required_if:role,agent_14|exists:departments,id',
                'manager'=>'nullable|required_if:role,supervisor_13|exists:members,id',
                'description'=>'nullable|max:2999',

            ]);

            try
            {
                $member = new member();
                $member->externalid = "member".time();
                $member->name = $req->name;
                $member->email = $req->email;
                $member->phone = $req->phone;
                $member->description = $req->description;

                $member->password = "password";
                $role = role::where("role_code",$req->role)->first();
                $member->role_id = $role->id;


                if($req->role == "agent_14")
                {

                    $dep = department::find($req->department);
                    $member->department_id = $req->department;
                    $member->supervisor_id = $dep->get_supervisor->id ?? null;
                    $sup = member::find($member->supervisor_id);

                }
                if($req->role == "supervisor_13")
                {
                    $sup = member::find($req->manager);
                    $member->supervisor_id = $req->manager;
                }
                if($member->save())
                {
                    if(isset($sup)){
                    $desc = "A new ".$role->name." ( ".$member->name." ) has been added under the supervison of ".$sup->name." by ".session("cms_member_name")." (".session("cms_member_role_name").")";
                    }
                    else
                    {
                        $desc = "A new ".$role->name." ( ".$member->name." ) has been added by ".session("cms_member_name")." (".session("cms_member_role_name").")";
                    }
                    $users = array();
                    $this->mail_subject ="New ".$role->name." Added";
                    $this->mail_body['description'] = $desc;
                    $this->mail_link = "member/".$member->externalid;
                    $users = $this->get_mail_members($this->mail_subject, $this->mail_body, $this->mail_link, ['manager_12','super_admin_11'],[],$users);
                    if($req->role == "agent_14")
                    {
                        $users = $this->get_specific_member($this->mail_subject, $this->mail_body, $this->mail_link, $member->supervisor_id,$users);
                    }
                    $this->mail_subject ="Your Account Created";
                    $this->mail_body['description'] = "Your account as ".$role->name." has been created in CMS with email:".$member->email." and password: password";
                    $this->mail_link = "login";
                    $users = $this->get_specific_member($this->mail_subject, $this->mail_body, $this->mail_link, $member->id,$users);

                     dispatch(
                        function () use ($users)
                        {
                            $this->send_to_members("","","",$users,"false","2");
                        }
                    )->delay(now()->addSeconds(2));
                        session()->flash("form_submit_flag","true");
                        session()->flash("form_submit_msg",'Member Added Successfully');

                        return redirect()->back();
                }
            }
            catch(Exception $e)
            {
                return redirect()->back()->with(['form_submit_flag' => 'false', 'form_submit_msg' => 'An Exception Occured'.$e]);
            }
        }
        return view("member.add_member",compact("roles","departments","managers"));
    }

    public function add_manager(Request $req)
    {
    //    return config("app.queue_time");

        if($req->method() == "POST")
        {
            $req->validate([
                'name'=>'required|min:3|max:60',
                'email'=>'required|email|max:200',
                'phone'=>'required|min:10|max:15',
                'description'=>'nullable|max:2999',

            ]);

            try
            {
                $member = new member();
                $member->externalid = "member".time();
                $member->name = $req->name;
                $member->email = $req->email;
                $member->phone = $req->phone;
                $member->password = "password";

                $role = role::where("role_code","manager_12")->first();
                $member->role_id = $role->id;
                $member->supervisor_id = session("cms_member_id");

                if($member->save())
                {
                    $users = array();
                    $this->mail_subject ="New Manager Added";
                    $this->mail_body['description'] = "A new Manager named (".$member->name.") has been added by ".session("cms_member_name")." (".session("cms_member_role_name").")";
                    $this->mail_link = "view_all_members";
                    $users = $this->get_mail_members($this->mail_subject, $this->mail_body, $this->mail_link, ['super_admin_11'],[],$users);

                    $this->mail_subject ="Your Account Created";
                    $this->mail_body['description'] = "Your account as a manager has been created in CMS with email:".$member->email." and password: password";
                    $this->mail_link = "login";
                    $users = $this->get_specific_member($this->mail_subject, $this->mail_body, $this->mail_link, $member->id,$users);
                    // return $this->send_to_members("","","",$users,"false","2");

                     dispatch(
                        function () use ($users)
                        {
                            $this->send_to_members("","","",$users,"false","2");
                        }
                    )->delay(now()->addSeconds(config("app.queue_time")));
                    return redirect()->back()->with(['form_submit_flag' => 'true', 'form_submit_msg' => 'Manager Added Successfully']);
                }
            }
            catch(Exception $e)
            {
                return redirect()->back()->with(['form_submit_flag' => 'false', 'form_submit_msg' => 'An Exception Occured'.$e->getMessage()]);
            }
        }
        return view("member.add_manager");
    }

    public function login(Request $req)
    {
        if($req->method() == "POST")
        {
            $req->validate([
                'email' => 'required|email',
                'password' => 'required|max:20'
            ]);

           $user = member::where("email",$req->email)->where("password",$req->password)->with("get_role:id,name,role_code")->first();
            if ($user) {
                $req->session()->put("cms_member_id", $user->id);
                $req->session()->put("cms_member_name", $user->name);
                $req->session()->put("cms_member_role", (($user->get_role->role_code) ?? ''));
                $req->session()->put("cms_member_role_name", (($user->get_role->name) ?? ''));
                if($req->redirect_to && $req->redirect_to!="")
                {
                    return redirect(config("app.domain").$req->redirect_to);
                }
                return redirect()->route("dashboard");


            }
            else {
                return redirect()->back()->with(["login_msg" => 'Invalid Credentials']);
            }
        }
        return view("login");
    }

    public function logout(Request $req)
    {
        $req->session()->pull("cms_member_id");
        $req->session()->pull("cms_member_name");
        $req->session()->pull("cms_member_role");
        return redirect()->route("login");
    }

    public function my_members()
    {
        $members = member::with("get_role:id,name")->where("supervisor_id",session("cms_member_id"))->with("get_supervisor:id,name")->paginate(config("app.paginate"));
        return view("member.my_members",compact("members"));

    }

    public function view_all_members(Request $req)
    {
        $companies = array();
        if ($req->id && $req->id != "") {

            $members = member::where('externalid', 'LIKE', '%' . $req->id . '%')->whereRelation("get_role","role_code","!=","super_admin_11")->paginate(config("app.paginate"));
            $roles=role::all();
            return view("member.view_all_members",compact("members","roles"));

        } elseif ($req->name && $req->name != "") {

            $members = member::where('name', 'LIKE', '%' . $req->name . '%')->whereRelation("get_role","role_code","!=","super_admin_11")->orderBy("id", "desc")->paginate(config("app.paginate"));
            $roles=role::all();
            return view("member.view_all_members",compact("members","roles"));

        }
        $members = member::with("get_role:id,name")->with("get_supervisor:id,name")->whereRelation("get_role","role_code","!=","super_admin_11")->orderBy("id","desc")->paginate(config("app.paginate"));
        $roles=role::all();
        return view("member.view_all_members",compact("members","roles"));
    }

    public function edit_member(Request $req,$id)
    {
        if($req->method()=="POST")
        {

            $req->validate([
                'member_id'=>'required|exists:members,id',
                'name'=>'required',
                'phone'=>'required',
                'email'=>'nullable|unique:members,email,'.$req->member_id,
                'description'=>'nullable|max:2999',

            ]);

            $member = member::find($req->member_id);
                $member->name = $req->name;


                // if($req->email == "")
                // {
                //     $member->email = "email_released_".$member->id;
                // }
                // else
                // {
                //     $member->email = $req->email;
                // }
                $member->email = $req->email;
                $member->description = $req->description;


                $member->phone = $req->phone;

                if($member->save())
                {
                    if($member->wasChanged())
                    {
                        $users = array();
                        $this->mail_subject ="Your Information Updated";
                        $this->mail_body['description'] = "Your Information has been Updated Check it Out.";
                        $this->mail_link = "login";
                        $users = $this->get_specific_member($this->mail_subject, $this->mail_body, $this->mail_link, $req->member_id,$users);
                         dispatch(
                            function () use ($users)
                            {
                                $this->send_to_members("","","",$users,"false","2");
                            }
                        )->delay(now()->addSeconds(config("app.queue_time")));
                        return redirect()->back()->with(['form_submit_flag' => 'true', 'form_submit_msg' => 'Changes Saved Successfuly']);
                    }
                    else
                    {
                        return redirect()->back()->with(['form_submit_flag' => 'false', 'form_submit_msg' => 'No Data Changed']);
                    }
                }
                else{
                    return redirect()->back()->with(['form_submit_flag' => 'false', 'form_submit_msg' => 'Something Went Wrong. Please try Again']);
                }
        }
        $member=member::find($id);
        if($member)
        {
            return view('member.edit_member',['member'=>$member]);
        }
        else{
            return redirect()->back()->with(['form_submit_flag' => 'false', 'form_submit_msg' => 'Member Not Found']);
        }
    }

    public function forget_password(Request $req)
    {
        if($req->method()=="POST")
        {
            $req->validate([
                'email'=>'required|exists:members,email',
            ]);

            $member=member::where('email',$req->email)->first();
            if($member)
            {
                $pass = substr(str_shuffle('0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ'), 1, 8);
                $member->password=$pass;
                $member->save();

                $this->mail_users =array();
                $users = array();
                $this->flag = "member_otp";
                $this->mail_subject = "Password Reset";
                $this->mail_body['description'] = "This is your new password <br><span style='font-size:16px'>" . $pass."</span> <br>";
                $this->mail_link = "login";
                $users = $this->get_specific_member($this->mail_subject, $this->mail_body, $this->mail_link, $member->id,$users);
                $this->send_to_members("","","",$users,"false","2");

                session()->flash("otp_msg", "New Password has been sent to your email.");
                return redirect()->route('login');
            }
            else
            {
                session()->flash("otp_msg", "No Account belongs to this email.");
                return redirect()->route('company-login');
            }
        }

        return view('member.forget_password');
    }
}
