<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Models\ticket;
use App\Models\member;
use App\Models\company;
use Exception;
use App\Models\tag;
use App\Models\company_tag;
use App\Models\department;
use App\Http\Traits\SystemMail;
use App\Models\ticket_note;
use App\Models\ticket_chat;
use Illuminate\Support\Facades\DB;

class TicketController extends Controller
{
    use SystemMail;

    public function my_department_tickets(Request $req)
    {
        $mem = member::find(session("cms_member_id"));
        $tickets_count = ticket::where("agent_id",session("cms_member_id"))->whereDate("agent_assigned_at",date("Y-m-d"))->count();
        $filters = array();
        $query = ticket::query();
        if($req->input("ticket_id"))
        {
            $filters['ticket_id'] = $req->ticket_id;
            $query->where("externalid",$req->ticket_id);
        }
        if($req->input("customer"))
        {
            $filters['customer'] = $req->input("customer");

            $query->whereRelation("get_company","email","=",$req->customer);
        }
        if($req->input("status") && $req->input("status")!="")
        {
            if($req->status == "pending")
            {
                $query = $query->where(function($q)
                {
                    $q->whereNull("status")->orWhere("status","pendning");
                });
            }
            else{
            $query = $query->where("status",$req->input("status"));}
            $filters['status'] = $req->status;

        }

        if($req->input("assignment") && $req->input("assignment")!="")
        {
            if($req->assignment == "assigned")
            {
                $query = $query->whereNotNull("agent_id");
            }
            else if($req->assignment == "not_assigned")
            {
                $query = $query->whereNull("agent_id");
            }

            $filters['assignment'] = $req->assignment;
        }
        $customers = company::select("id","externalid","name","email")->get();

        $tickets = $query->with("get_company","get_agent")->whereNotNull("department_id")->where("department_id", $mem->department_id)->orderBy("id","desc")->paginate(config("app.paginate"));
        return view("tickets.my_department_tickets", compact("tickets","tickets_count","filters","customers"));
    }

    public function pick_ticket(Request $req)
    {
    //   return $req->all();
        $ticket = ticket::where("externalid",$req->ticket_id)->first();

        if($ticket)
        {
            $ticket->agent_id = session("cms_member_id");
            $ticket->agent_assigned_at = date("Y-m-d H:i:s");
            try{
            if($ticket->save())
                {
                    return redirect()->back()->with(['form_submit_flag' => 'true', 'form_submit_msg' => 'Ticket No '.$ticket->externalid.' has been picked up successfully']);
                }
                else
                {
                    return redirect()->back()->with(['form_submit_flag' => 'false', 'form_submit_msg' => 'Ticket was not picked up']);

                }
            }
            catch(Exception $e)
            {
                return redirect()->back()->with(['form_submit_flag' => 'false', 'form_submit_msg' => 'An Exception Occured'.$e->getMessage()]);
            }
        }
        else
        {
            return redirect()->back()->with(['form_submit_flag' => 'false', 'form_submit_msg' => 'Ticket Not Found']);
        }

    }

    public function generate_ticket(Request $req)//org
    {

        if($req->method() == "POST")
        {
            $req->validate([
                'subject' => 'required|min:2|max:60',
                'department' => 'required|exists:departments,id',
                'customer' => 'required|exists:companies,id'
            ]);
            $this->mail_users = array();
            $department = department::find($req->department);
            $customer = company::find($req->customer);
            $ticket = new ticket();
            $ticket->subject = $req->subject;
            $ticket->department_id = $req->department;
            $ticket->externalId = time();
            $ticket->company_id = $req->customer;
            $ticket->manager_id = ($department->get_supervisor->get_supervisor->id ?? null);
            if ($ticket->manager_id != "")
            {
                $ticket->manager_assigned_at = date('Y:m:d H:i:s');
            }
            $ticket->supervisor_id = ($department->get_supervisor->id ?? null);
            if ($ticket->supervisor_id != "") {
                $ticket->supervisor_assigned_at = date('Y:m:d H:i:s');
            }
            $ticket->status = "pending";
            if($department->ticket_time!="" && $department->ticket_time>0)
            {
                $ticket->time = $department->ticket_time;
            }
            else
            {
                $ticket->time = 1440;
            }
            $ticket->internal_status = "pending";
            $ticket->source = "organization";
            $ticket->generated_by = session("cms_member_id");
            try{
            if ($ticket->save()) {

                $ticket->expire_at = date("Y-m-d H:i:s", strtotime("+".$ticket->time." minute", strtotime($ticket->created_at)));
                $ticket->time_set_at = date("Y-m-d H:i:s");
                $ticket->save();
                $users = array();
                $this->mail_subject ="New Ticket Generated";
                $this->mail_body['description'] = "A new ticket has been generated for the customer <b>".$customer->name." (".$customer->email.")</b>  against department (".$department->name.")";
                $this->mail_link = "ticket_detail/".$ticket->id;
                $users = $this->get_mail_members($this->mail_subject, $this->mail_body, $this->mail_link, ['super_admin_11'],[],$users);
                $users = $this->get_specific_member($this->mail_subject, $this->mail_body, $this->mail_link, $ticket->supervisor_id,$users);
                $users = $this->get_specific_member($this->mail_subject, $this->mail_body, $this->mail_link, $ticket->manager_id,$users);
                $this->mail_link = "customer_chat/".$ticket->externalid;
                $this->mail_link2 = config("app.cust_domain").$this->mail_link;
                dispatch(
                        function () use ($users,$ticket,$department)
                        {
                            $this->mail_body['description'] = "A new ticket has been generated for you against department (".$department->name.")";
                            $this->mail_link = "customer_chat/".$ticket->externalid;
                            $this->mail_link2 = config("app.cust_domain").$this->mail_link;
                            $this->send_to_members("","","",$users,"false","2");
                            $this->send_to_customer( $this->mail_subject, $this->mail_body, $this->mail_link, $this->mail_link2,$ticket->company_id,"2");
                        }
                )->delay(now()->addSeconds(config("app.queue_time")));

                return redirect()->back()->with(['form_submit_flag' => 'true', 'form_submit_msg' => 'Ticket Generated Successfully']);

            }
        }
        catch(Exception $e)
        {
            return redirect()->route('ticket_detail',$ticket->id)->with(['form_submit_flag' => 'false', 'form_submit_msg' => 'An Exception Occured.'.$e->getMessage()]);
        }
        }
        $dep = department::select("id","name")->get();
        $customers = company::select("id","email","name")->get();
        return view("tickets.generate_ticket", compact("dep","customers"));

    }


    public function create_ticket(Request $req)
    {
        if ($req->method() == "POST") {

            if (session('cms_company_id')) {
                $req->validate([
                    'subject' => 'required|min:2|max:60',
                    'department' => 'required|exists:departments,id',
                ]);
            } else {
                $req->validate([
                    'name' => 'required|min:2|max:60',
                    'email' => 'required',
                    'phone' => 'required',
                    'subject' => 'required|min:2|max:60',
                    'department' => 'required|exists:departments,id',
                ]);
            }

            //return $req->all();

            try {
                $comp_id="";
                if (session('cms_company_id') == "") {
                    $comp = new company();
                    $comp->externalid = "company" . time();
                    $comp->name = $req->name;
                    $comp->email = $req->email;
                    $comp->phone = $req->phone;
                    $comp->status = "onhold";
                    //$comp->password = $req->password;
                    if ($comp->save()) {
                         $comp_id=$comp->id;
                         $req->session()->put("cms_company_email", $comp->email);
                    }
                }
                else{
                    $comp_id=session('cms_company_id');
                }
                $comp=company::find($comp_id);
                $this->mail_users = array();
                $department = department::find($req->department);
                $ticket = new ticket();
                $ticket->subject = $req->subject;
                $ticket->department_id = $req->department;
                $ticket->externalId = time();
                $ticket->company_id = $comp_id;
                $ticket->manager_id = ($department->get_supervisor->get_supervisor->id ?? null);
                if ($ticket->manager_id != "") {
                    $this->mail_users[] = array("user_id" => $ticket->manager_id);
                    $ticket->manager_assigned_at = date('Y:m:d H:i:s');
                }
                $ticket->supervisor_id = ($department->get_supervisor->id ?? null);
                if ($ticket->supervisor_id != "") {
                    $this->mail_users[] = array("user_id" => $ticket->supervisor_id);

                    $ticket->supervisor_assigned_at = date('Y:m:d H:i:s');
                }
                $ticket->status = "pending";
                if($department->ticket_time!="" && $department->ticket_time>0)
                {
                    $ticket->time = $department->ticket_time;
                }
                else
                {
                    $ticket->time = 1440;
                }
                $ticket->internal_status = "pending";
                $ticket->source = "customer";

                if ($ticket->save()) {

                    $ticket->expire_at = date("Y-m-d H:i:s", strtotime("+1440 minute", strtotime($ticket->created_at)));
                    $ticket->time_set_at = date("Y-m-d H:i:s");
                    $ticket->save();

                    $this->mail_subject = "New Ticket Generated";
                    $this->mail_body['description'] = "A new ticket has been generated for department (" . $department->name . ")";
                    $this->mail_link = "ticket_detail/" . $ticket->id;


                    if ($comp->status == "onhold") {
                        $otp = substr(str_shuffle('0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ'), 1, 6);
                        $this->mail_users =array();
                        $this->mail_users[] = array("user_id" => $comp_id);
                        $this->flag = "otp";
                        $this->mail_subject = "Verify Your Email";
                        $this->mail_body['description'] = "A new ticket has been generated for department (" . $department->name . "). Please Verify Your email to track your tickets. <br> Verification Code is: " . $otp;
                        $this->mail_link = "/home";

                        $user = company::find($comp_id);
                        $user->otp = $otp;
                        $current_time = date('Y:m:d H:i:s');
                        $newDate = date('Y:m:d H:i:s', strtotime($current_time . ' +40 minutes'));
                        $user->otp_expiry = $newDate;
                        $user->save();
                          $this->send_mail($this->mail_subject, $this->mail_body, $this->mail_link, $this->mail_users, $this->flag);
                        $msg = " Please Verify Your email to track your tickets. OTP has been sent to your email for verification";
                        session()->flash("otp_msg", $msg);

                        $users = array();
                        $this->mail_subject ="New Ticket Generated";
                        $this->mail_body['description'] = "A new ticket has been generated from the department (".$department->name.")";
                        $this->mail_link = "ticket_detail/".$ticket->id;
                        $users = $this->get_mail_members($this->mail_subject, $this->mail_body, $this->mail_link, ['super_admin_11'],[],$users);
                        $users = $this->get_specific_member($this->mail_subject, $this->mail_body, $this->mail_link, $ticket->supervisor_id,$users);
                        $users = $this->get_specific_member($this->mail_subject, $this->mail_body, $this->mail_link, $ticket->manager_id,$users);

                        dispatch(
                            function () use ($users)
                            {
                                $this->send_to_members("","","",$users,"false","2");
                            }
                        )->delay(now()->addSeconds(config("app.queue_time")));





                        return redirect()->route('verify_otp');
                    } else {
                        $users = array();
                        $this->mail_subject ="New Ticket Generated";
                        $this->mail_body['description'] = "A new ticket has been generated from the department (".$department->name.")";
                        $this->mail_link = "ticket_detail/".$ticket->id;
                        $users = $this->get_mail_members($this->mail_subject, $this->mail_body, $this->mail_link, ['super_admin_11'],[],$users);
                        $users = $this->get_specific_member($this->mail_subject, $this->mail_body, $this->mail_link, $ticket->supervisor_id,$users);
                        $users = $this->get_specific_member($this->mail_subject, $this->mail_body, $this->mail_link, $ticket->manager_id,$users);

                        dispatch(
                            function () use ($users)
                            {
                                $this->send_to_members("","","",$users,"false","2");
                            }
                        )->delay(now()->addSeconds(config("app.queue_time")));
                        return redirect()->route('my_tickets')->with(['form_submit_flag' => 'true', 'form_submit_msg' => 'Ticket Generated Successfully']);
                    }
                }
            } catch (Exception $e) {
                return redirect()->back()->with(['form_submit_flag' => 'false', 'form_submit_msg' => 'An Exception Occured' . $e->getMessage()]);
            }
        }
        $dep = department::get();
        return view('users.create_ticket', compact("dep"));
    }

    public function close_ticket(Request $req)
    {
        $ticket = ticket::where("externalid",$req->ticket_id)->where("status","!=","closed")->first();
        if(!$ticket)
        {
            return redirect()->back()->with(['form_submit_flag' => 'false', 'form_submit_msg' => 'Ticket Not Found']);
        }
        $ticket->closed_by = session("cms_member_id");
        $ticket->closed_at = date("Y-m-d H:i:s");
        $ticket->close_mode = "manual";
        $ticket->status = "closed";

        if($ticket->save())
        {
            $users = array();
            $this->mail_subject ="Ticket Closed";
            $this->mail_body['description'] = "The Ticket #".$ticket->externalid." has been closed";
            $this->mail_link = "ticket_detail/".$ticket->id;
            $users = $this->get_mail_members($this->mail_subject, $this->mail_body, $this->mail_link, ['super_admin_11'],[],$users);
            $users = $this->get_specific_member($this->mail_subject, $this->mail_body, $this->mail_link, $ticket->supervisor_id,$users);
            $users = $this->get_specific_member($this->mail_subject, $this->mail_body, $this->mail_link, $ticket->manager_id,$users);
            $users = $this->get_specific_member($this->mail_subject, $this->mail_body, $this->mail_link, $ticket->agent_id,$users);
            $this->mail_link = "customer_chat/".$ticket->externalid;

            $this->mail_link2 = config("app.cust_domain").$this->mail_link;

            dispatch(
                function () use ($users,$ticket)
                {
                    $this->send_to_members("","","",$users,"false","2");
                    $this->mail_link = "customer_chat/".$ticket->externalid;

                    $this->mail_link2 = config("app.cust_domain").$this->mail_link;
                    $this->send_to_customer($this->mail_subject, $this->mail_body, $this->mail_link, $this->mail_link2,$ticket->company_id,"2");
                }
            )->delay(now()->addSeconds(config("app.queue_time")));
            return redirect()->back()->with(['form_submit_flag' => 'true', 'form_submit_msg' => 'Ticket Closed Successfully']);
        }


    }
    public function my_tickets()
    {
        $tickets=ticket::where('company_id',session('cms_company_id'))->with("get_department","get_agent")->orderBy("id","desc")->get();
        return view('users.view_tickets',compact("tickets"));
    }

    public function create_ticket_customer(Request $req)
    {
        if ($req->method() == "POST") {


                $req->validate([
                    'name' => 'required|min:2|max:60',
                    'email' => 'required|unique:companies,email,'.$req->email,
                    'phone' => 'required',
                    'subject' => 'required|min:2|max:60',
                    'department' => 'required|exists:departments,id',
                ]);


            //return $req->all();

            try {

                    $comp = new company();
                    $comp->externalid = "company" . time();
                    $comp->name = $req->name;
                    $comp->email = $req->email;
                    $comp->phone = $req->phone;
                    $comp->password = "password";
                    $comp->status = "active";
                    //$comp->password = $req->password;
                    if ($comp->save()) {
                       $msg="Customer has been created";
                    }


                $this->mail_users = array();
                $department = department::find($req->department);
                $ticket = new ticket();
                $ticket->subject = $req->subject;
                $ticket->department_id = $req->department;
                $ticket->externalId = time();
                $ticket->company_id = $comp->id;
                $ticket->manager_id = ($department->get_supervisor->get_supervisor->id ?? null);
                if ($ticket->manager_id != "") {
                    $this->mail_users[] = array("user_id" => $ticket->manager_id);
                    $ticket->manager_assigned_at = date('Y:m:d H:i:s');
                }
                $ticket->supervisor_id = ($department->get_supervisor->id ?? null);
                if ($ticket->supervisor_id != "") {
                    $this->mail_users[] = array("user_id" => $ticket->supervisor_id);

                    $ticket->supervisor_assigned_at = date('Y:m:d H:i:s');
                }
                $ticket->status = "pending";
                if($department->ticket_time!="" && $department->ticket_time>0)
                {
                    $ticket->time = $department->ticket_time;
                }
                else
                {
                    $ticket->time = 1440;
                }
                $ticket->internal_status = "pending";
                $ticket->source = "organization";

                if ($ticket->save()) {

                    $ticket->expire_at = date("Y-m-d H:i:s", strtotime("+1440 minute", strtotime($ticket->created_at)));
                    $ticket->time_set_at = date("Y-m-d H:i:s");
                    $ticket->save();

                    // $this->mail_subject = "New Ticket Generated";
                    // $this->mail_body['description'] = "A new ticket has been generated for department (" . $department->name . ")";
                    // $this->mail_link = "ticket_detail/" . $ticket->id;




                        $this->mail_users =array();
                        $this->mail_users[] = array("user_id" => $comp->id);
                        $this->flag = "otp";
                        $this->mail_subject = "New Ticket Generated";
                        $this->mail_body['description'] = "A new ticket has been generated for department (" . $department->name . "). Looking for further detail? <br>Please use these login credentials <br> Email:  " . $comp->email."<br>Password: ".$comp->password;
                        $this->mail_link = "customer_chat/".$ticket->externalid;
                        $this->mail_link2 = config("app.cust_domain").$this->mail_link;



                        dispatch(
                            function () use ($ticket)
                            {

                               $this->send_to_customer( $this->mail_subject, $this->mail_body, $this->mail_link, $this->mail_link2,$ticket->company_id,"2");
                            }
                        )->delay(now()->addSeconds(config("app.queue_time")));
                          //$this->send_mail($this->mail_subject, $this->mail_body, $this->mail_link, $this->mail_users, $this->flag);
                        $msg = " A new ticket has been generated. ".$msg;

                        $users = array();
                        $this->mail_subject ="New Ticket Generated";
                        $this->mail_body['description'] = "A new ticket has been generated from the department (".$department->name.")";
                        $this->mail_link = "ticket_detail/".$ticket->id;
                        $users = $this->get_mail_members($this->mail_subject, $this->mail_body, $this->mail_link, ['super_admin_11'],[],$users);
                        $users = $this->get_specific_member($this->mail_subject, $this->mail_body, $this->mail_link, $ticket->supervisor_id,$users);
                        $users = $this->get_specific_member($this->mail_subject, $this->mail_body, $this->mail_link, $ticket->manager_id,$users);

                        dispatch(
                            function () use ($users)
                            {
                                $this->send_to_members("","","",$users,"false","2");
                            }
                        )->delay(now()->addSeconds(config("app.queue_time")));

                        return redirect()->route("ticket_detail",$ticket->id)->with(['form_submit_flag' => 'true', 'form_submit_msg' => $msg]);

                }
                else{
                    return redirect()->back()->with(['form_submit_flag' => 'false', 'form_submit_msg' => 'SOmething went wrong. Please Try again later' ]);
                }
            } catch (Exception $e) {
                return redirect()->back()->with(['form_submit_flag' => 'false', 'form_submit_msg' => 'An Exception Occured' . $e->getMessage()]);
            }
        }
        $dep = department::get();
        return view('tickets.create_ticket_customer', compact("dep"));
    }
    public function view_tickets(Request $req)
    {
        $role = "";
        $tickets_count="";
        $query = ticket::query();
        $params = "";
        $agents = [];
        $filters = array();
        $departments = department::with("get_supervisor")->get();
        if($req->input("department"))
        {
            $req->validate(
                ['department' => 'required|exists:departments,externalid']
            );
            $params = $req->department;
            $agents = member::whereRelation("get_department","externalid","=",$req->department)->get();
            $query = $query->whereRelation("get_department","externalid","=",$req->department);
            $filter['department'] = $req->department;

        }
        if($req->input("ticket_id"))
        {
            $query = $query->where("externalid",$req->input("ticket_id"));
            $filters['ticket_id'] = $req->ticket_id;
        }
        if($req->input("status") && $req->input("status")!="")
        {
            if($req->status == "pending")
            {
                $query = $query->where(function($q)
                {
                    $q->whereNull("status")->orWhere("status","pendning");
                });
            }
            else{
            $query = $query->where("status",$req->input("status"));}
            $filters['status'] = $req->status;

        }

        if($req->input("assignment") && $req->input("assignment")!="")
        {
            if($req->assignment == "assigned")
            {
                $query = $query->whereNotNull("agent_id");
            }
            else if($req->assignment == "not_assigned")
            {
                $query = $query->whereNull("agent_id");
            }

            $filters['assignment'] = $req->assignment;
        }
        if(session("cms_member_role") == "super_admin_11")
        {
            // $departments = department::with("get_supervisor")->get();

            $role = "Super Admin";
            $tickets = $query->with("get_company")->with("get_department")->orderBy("id","desc")->paginate(config("app.paginate"));

            // $tickets = ticket::with("get_company")->with("get_department")->get();
        }

        else if(session("cms_member_role") == "manager_12")
        {
            // $departments = department::with("get_supervisor")->get();

            $role = "Manager";
            $tickets_count = ticket::where("manager_id",session("cms_member_id"))->whereDate("manager_assigned_at",date("Y-m-d"))->count();
            $tickets = $query->with("get_company")->where("manager_id",session("cms_member_id"))->with("get_department")->orderBy("id","desc")->paginate(config("app.paginate"));
        }

        else if(session("cms_member_role") == "supervisor_13")
        {
            $departments = department::with("get_supervisor")->where("supervisor_id",session("cms_member_id"))->get();

            $role = "Supervisor";
            $tickets_count = ticket::where("supervisor_id",session("cms_member_id"))->whereDate("supervisor_assigned_at",date("Y-m-d"))->count();
            $tickets = $query->with("get_company")->where("supervisor_id",session("cms_member_id"))->with("get_department")->orderBy("id","desc")->paginate(config("app.paginate"));
        }

        else if(session("cms_member_role") == "agent_14")
        {
            $role = "Agent";
            $tickets_count = ticket::where("agent_id",session("cms_member_id"))->whereDate("agent_assigned_at",date("Y-m-d"))->count();
            $tickets = $query->with("get_company")->where("agent_id",session("cms_member_id"))->with("get_department")->orderBy("id","desc")->paginate(config("app.paginate"));
        }
        else
        {
            return ['message' => 'Unknown Role'];
        }
        // return $filters;
       return view("tickets.view_tickets", compact("tickets","role","tickets_count","params","agents","departments","filters"));
    }

    public function all_tickets(Request $req)
    {
        $filters = array();
        $query = ticket::query();
        if($req->input("ticket_id"))
        {
            $filters['ticket_id'] = $req->ticket_id;
            $query->where("externalid",$req->ticket_id);
        }
        if($req->input("customer"))
        {
            $filters['customer'] = $req->customer;

            $query->whereRelation("get_company","email","=",$req->customer);
        }
        if($req->input("status") && $req->input("status")!="")
        {
            if($req->status == "pending")
            {
                $query = $query->where(function($q)
                {
                    $q->whereNull("status")->orWhere("status","pendning");
                });
            }
            else{
            $query = $query->where("status",$req->input("status"));}
            $filters['status'] = $req->status;

        }

        if($req->input("assignment") && $req->input("assignment")!="")
        {
            if($req->assignment == "assigned")
            {
                $query = $query->whereNotNull("agent_id");
            }
            else if($req->assignment == "not_assigned")
            {
                $query = $query->whereNull("agent_id");
            }

            $filters['assignment'] = $req->assignment;
        }
        if($req->input("department"))
        {
            $filters['department'] = $req->department;

            $query->whereRelation("get_department","externalid","=",$req->department);
        }
        $departments = department::select("id","externalid","name")->get();
        $customers = company::select("id","externalid","name","email")->get();

        $tickets = $query->with("get_company")->with("get_department","get_agent")->orderBy("id","desc")->paginate(config("app.paginate"));
        return view("tickets.all_tickets", compact("tickets","filters","customers","departments"));

    }

    public function submit_multiple_tickets(Request $req)
    {
        // return $deps = department::where("supervisor_id",session("cms_member_id"))->pluck("id");
        $req->validate(
            ['tickets'=>'required|array|min:1|max:30',
            'tickets.*'=>'exists:tickets,externalid']
        );
         $tickets = ticket::whereIn("externalid",$req->tickets)->get();
         $agent = member::where("id",$req->agent)->first();

         if($agent)
         {
            if($agent->get_agent_tickets()->saveMany($tickets))
            {
                ticket::whereIn("externalid",$req->tickets)->update(['agent_assigned_at'=>date("Y-m-d")]);
                $users = array();
                $this->mail_subject ="Multiple Tickets Assigned To Employee";
                $this->mail_body['description'] = sizeof($tickets) ." tickets have been assigned to agent (".$agent->name.")";
                $this->mail_link = "view_tickets";
                $users = $this->get_mail_members($this->mail_subject, $this->mail_body, $this->mail_link, ['super_admin_11'],[],$users);
                // $users = $this->get_specific_member($this->mail_subject, $this->mail_body, $this->mail_link, $agent->id,$users);

                $this->mail_body['description'] = sizeof($tickets) ." tickets have been assigned you";
                $users = $this->get_specific_member($this->mail_subject, $this->mail_body, $this->mail_link, $agent->id,$users);
                dispatch(
                    function () use ($users)
                    {
                        $this->send_to_members("","","",$users,"false","2");
                    }
                )->delay(now()->addSeconds(config("app.queue_time")));

                return redirect()->back()->with(['form_submit_flag' => 'true', 'form_submit_msg' => "Tickets Assigned Successfully"]);
            }
            else
            {
                return redirect()->back()->with(['form_submit_flag' => 'false', 'form_submit_msg' => "Tickets was not Assigned. Try Again"]);
            }
         }
         return redirect()->back()->with(['form_submit_flag' => 'false', 'form_submit_msg' => "Agent Or Department Not Found"]);

    }

    public function ticket_detail($id)
    {
       $managers = [];$departments = [];$agents = [];
       $ticket = ticket::with("get_company")->with("get_department")->with("get_manager","get_supervisor","get_agent")->where("id",$id)->first();
        $params="1";
       if(!$ticket)
       {
            return ['message' => 'Ticket Not Found'];
       }


       if(session("cms_member_role") != "super_admin_11" && session("cms_member_id") != $ticket->manager_id && session("cms_member_id") != $ticket->supervisor_id && session("cms_member_id") != $ticket->agent_id)
        {
            $params ="0";
            // return view("lock_pages.lock_page",['message'=>'Unauthorized access']);
        }
        if(session("cms_member_role") == "manager_12" || session("cms_member_role") == "super_admin_11" || session("cms_member_role") != "super_admin_11")
        {
             $departments = department::with("get_supervisor")->get();
        }

       if(session("cms_member_role") == "super_admin_11")
       {
            $managers = member::whereRelation("get_role","role_code","=","manager_12")->get();
       }
       else if(session("cms_member_role") == "supervisor_13")
       {
            $agents = member::whereRelation("get_role","role_code","=","agent_14")->where("department_id",$ticket->department_id)->where("supervisor_id",session("cms_member_id"))->withCount("get_agent_today_tickets")->get();
       }


       return view("tickets.ticket_detail", compact("ticket","managers","departments","agents"));
    }

    public function submit_ticket_detail(Request $req)
    {
        $ticket = ticket::where("externalid",$req->ticket_id)->where("status","!=","closed")->first();
        if(!$ticket)
        {
            return redirect()->back()->with(['form_submit_flag' => 'false', 'form_submit_msg' => "Ticket Not Found or Might have been closed"]);

        }
        $ticket_backup = $ticket;
        $desc="";$cust_desc = "";
        $dep_val="nullable";
        $users = array();
        $this->mail_subject ="Ticket #".$ticket->externalid." has been updated";
        $this->mail_body['description'] = "";
        if($req->department!="" && $req->department!=$ticket->department_id)
        {
            $dep_val = "required";
        }
        $check=0;
        $check2=0;

        if(session("cms_member_role") == "super_admin_11")
        {
            $req->validate(
                ['manager' => 'required|exists:members,id',
                    'time' =>'required|integer|min:1|max:50000',
                    'department' => 'required|exists:departments,id',
                    'department_note'=>$dep_val."|max:5000",
                ]
            );
            $check=1;
            $check2=1;

            if($ticket->manager_id != $req->manager)
            {
                $ticket->manager_assigned_at = date("Y-m-d H:i:s");
                $ticket->manager_id = $req->manager;
                $ticket->supervisor_id = null;
                $ticket->agent_id = null;
                $ticket->status = "pending";
                $ticket->internal_status = "manager_assigned";
                $ticket->department_id = null;
            }




        }

       else if(session("cms_member_role") == "manager_12")
       {

        $req->validate(
            ['department' => 'required|exists:departments,id',
            'time' =>'required|integer|min:1|max:50000',
            'department_note'=>$dep_val."|max:5000",


            ]
        );
        $check=1;
        $check2=1;





       }

       else if(session("cms_member_role") == "supervisor_13")
       {
        $req->validate(
            ['agent' => 'required|exists:members,id',
            'department' => 'required|exists:departments,id',
            'time' =>'required|integer|min:1|max:50000',
            'department_note'=>$dep_val."|max:5000",

            ]
        );

        $check=1;
        $check2=1;
        if($ticket->agent_id != $req->agent)
        {
            $agent1 = member::find($ticket->agent_id);
            $agent2 = member::find($req->agent);

            $ticket->agent_assigned_at = date("Y-m-d H:i:s");
            $ticket->agent_id = $req->agent;
            $ticket->status = "pending";
            $ticket->internal_status = "agent_assigned";
            $desc = "Agent has been changed from ".($agent1->name ?? 'N/A') ." to ".(($agent2->name ?? 'N/A'))."";
            if($ticket_backup->agent_id!=""){

            $users = $this->get_specific_member($this->mail_subject, $this->mail_body, $this->mail_link, $ticket_backup->agent_id,$users);
            }
        }

        }
        else if(session("cms_member_role") == "agent_14")
        {
            $req->validate(
                [
                'time' =>'required|numeric|min:1|max:50000'
                ]);
                $check=1;
        }
        if($check2 == 1)
        {

            if($ticket->department_id != $req->department)
            {
                $dep = department::with("get_supervisor")->where("id",$req->department)->first();
                $dep1 = department::with("get_supervisor")->where("id",$ticket->department_id)->first();

                $ticket->supervisor_assigned_at = date("Y-m-d H:i:s");
                $ticket->department_id = $req->department;
                $ticket->supervisor_id = $dep->get_department->get_supervisor->name ?? null;
                $ticket->manager_assigned_at = date("Y-m-d H:i:s");
                $ticket->manager_id = $dep->get_department->get_supervisor->get_supervisor->name ?? null;
                $ticket->agent_id = null;
                $ticket->status = "pending";
                $ticket->internal_status = "supervisor_assigned";
                $desc = $desc.", Department has been changed from ".($dep1->name ?? 'N/A') ." to ".(($dep->name ?? 'N/A'))."";
                $cust_desc = $cust_desc." Department has been changed from ".($dep1->name ?? 'N/A') ." to ".(($dep->name ?? 'N/A'))."";
                if($ticket_backup->agent_id!=""){
                $users = $this->get_specific_member($this->mail_subject, $this->mail_body, $this->mail_link, $ticket_backup->agent_id,$users);}
                $users = $this->get_specific_member($this->mail_subject, $this->mail_body, $this->mail_link, $ticket_backup->supervisor_id,$users);

            }
        }
        if($check == 1)
        {
            if($ticket->time != $req->time)
            {

                $ticket->time = $req->time;
                $ticket->expire_at = date("Y-m-d H:i:s", strtotime("+".$req->time." minute", strtotime($ticket->created_at)));
                $ticket->time_set_by = session("cms_member_id");
                $ticket->time_set_at = date("Y-m-d H:i:s");
                $desc = $desc.", Time has been been updated to ".$req->time." minutes";
                $cust_desc = $cust_desc.", Time has been been updated to ".$req->time." minutes";
            }
            if($ticket->save())
            {
                if($check2 == 1)
                {
                    if($req->department_note!="")
                    {
                        $note  = new ticket_note();
                        $note->added_by = session("cms_member_id");
                        $note->ticket_id = $ticket->id;
                        $note->description = $req->department_note;
                        $note->subject = "department_note";
                        $note->save();
                    }
                }
                $this->mail_body['description'] =$desc;
                $this->mail_link = "ticket_detail/".$ticket->id;
                $users = $this->get_mail_members($this->mail_subject, $this->mail_body, $this->mail_link, ['manager_12'],[],$users);

                $users = $this->get_specific_member($this->mail_subject, $this->mail_body, $this->mail_link, $ticket->supervisor_id,$users);

                $users = $this->get_specific_member($this->mail_subject, $this->mail_body, $this->mail_link, $ticket->manager_id,$users);
                if($ticket->agent_id!="")
                {
                    $users = $this->get_specific_member($this->mail_subject, $this->mail_body, $this->mail_link, $ticket->agent_id,$users);
                }

                $this->mail_link = "customer_chat/".$ticket->externalid;
                $this->mail_link2 = config("app.cust_domain").$this->mail_link;


                dispatch(
                    function () use ($users,$cust_desc,$ticket)
                    {
                        $this->send_to_members("","","",$users,"false","2");

                        $this->mail_body['description'] = $cust_desc;
                        $this->mail_link = "customer_chat/".$ticket->externalid;
                        $this->mail_link2 = config("app.cust_domain").$this->mail_link;
                        $this->send_to_customer( $this->mail_subject, $this->mail_body, $this->mail_link, $this->mail_link2,$ticket->company_id,"2");

                    }
                )->delay(now()->addSeconds(config("app.queue_time")));



                return redirect()->back()->with(['form_submit_flag' => 'true', 'form_submit_msg' => "Ticket Updated Successfully"]);
            }
            else
            {
                return redirect()->back()->with(['form_submit_flag' => 'false', 'form_submit_msg' => "Ticket Not Updated"]);
            }
        }
    }

    public function ticket_chat(Request $req,$id)
    {
        $ticket = ticket::where("externalid",$id)->first();

        if(!$ticket)
        {
            // if(strtotime(date("Y-m-d H:i:s")) - strtotime($ticket->created_at) )
            return ['message' => 'Ticket Not Found Or You Do not have access to this ticket'];
        }
        $ticket = ticket::where("id",$ticket->id)->with("get_company")->first();
        if($req->method() == "POST")
        {
            if($req->type  == "open")
            {
                if($ticket->status == "pending")
                {
                    $ticket->status = "opened";
                    $ticket->opened_by = session("cms_member_id");
                    $ticket->opened_at = date("Y-m-d H:i:s");
                    if($ticket->save())
                    {
                        $users = array();
                        $this->mail_subject ="Ticket Opened";
                        $this->mail_body['description'] = "The Ticket #".$ticket->externalid." has been opened";
                        $this->mail_link = "ticket_detail/".$ticket->id;
                        $users = $this->get_mail_members($this->mail_subject, $this->mail_body, $this->mail_link, ['super_admin_11'],[],$users);
                        $users = $this->get_specific_member($this->mail_subject, $this->mail_body, $this->mail_link, $ticket->supervisor_id,$users);
                        $users = $this->get_specific_member($this->mail_subject, $this->mail_body, $this->mail_link, $ticket->manager_id,$users);
                        $users = $this->get_specific_member($this->mail_subject, $this->mail_body, $this->mail_link, $ticket->agent_id,$users);
                        $this->mail_link = "customer_chat/".$ticket->externalid;
                        $this->mail_link2 = config("app.cust_domain").$this->mail_link;

                        dispatch(
                            function () use ($users,$ticket)
                            {
                                $this->send_to_members("","","",$users,"false","2");
                                $this->mail_link = "customer_chat/".$ticket->externalid;
                                $this->mail_link2 = config("app.cust_domain").$this->mail_link;
                                $this->send_to_customer($this->mail_subject, $this->mail_body, $this->mail_link, $this->mail_link2,$ticket->company_id,"2");
                            }
                        )->delay(now()->addSeconds(config("app.queue_time")));

                        return redirect()->back()->with(['form_submit_flag' => 'true', 'form_submit_msg' =>"Ticket Opened. Go Down and Start Chat"]);
                    }
                }
                else
                {
                    return redirect()->back()->with(['form_submit_flag' => 'false', 'form_submit_msg' =>"Cannot Open Ticket due to unknown status"]);
                }
            }
        }
        return view("tickets.ticket_chat", compact("ticket"));
    }

    public function my_profile()
    {

        $company=company::where('id',session('cms_company_id'))->with('get_tags')->first();
       return view('users.company_profile',compact('company'));
    }


    public function ticket_notes(Request $req,$id,$action="")
    {
         $ticket = ticket::where("externalid",$id)->with("get_notes.get_adder.get_role")->first();
        if(!$ticket)
        {
            return ['message' => 'Ticket Not Found'];
        }

        if($action == "escalation")
        {
            $action = "escalation";
        }

        if($req->method() == "POST")
        {
            if($action == "escalation")
        {
            $req->validate([
                'description'=>'required|max:5000',
            'escalation' =>'required'
             ]);
        }
        else
        {
            $req->validate(['description'=>'required|max:5000']);
        }


            try{
            $note  = new ticket_note();
            $note->added_by = session("cms_member_id");
            $note->description = $req->description;
            if($req->action == "escalation")
            {
                $note->status = $req->escalation;
                if($req->escalation == "escalated")
                {
                    $ticket->escalation = $req->escalation;
                }
                else
                {
                    $ticket->escalation = null;
                }
                $ticket->save();
            }
            if($ticket->get_notes()->save($note))
            {
                $users = array();
                if($action == "escalation")
                {
                    $this->mail_subject ="Ticket ".$req->escalation;
                    $this->mail_body['description'] = " The Ticket # ".$ticket->externalid." has been ".$req->escalation."<br>".$req->description;
                }
                else
                {
                    $this->mail_subject ="New Ticket Note";
                    $this->mail_body['description'] = " A new note has been added against ticket # ".$ticket->externaldi.". <br>
                    ".$req->description."";
                }
                $this->mail_link = "ticket_notes/".$ticket->externalid;
                $users = $this->get_mail_members($this->mail_subject, $this->mail_body, $this->mail_link, ['super_admin_11'],[],$users);

                $users = $this->get_specific_member($this->mail_subject, $this->mail_body, $this->mail_link, $ticket->supervisor_id,$users);
                $users = $this->get_specific_member($this->mail_subject, $this->mail_body, $this->mail_link, $ticket->manager_id,$users);
                if($ticket->agent_id!="")
                {
                    $users = $this->get_specific_member($this->mail_subject, $this->mail_body, $this->mail_link, $ticket->agent_id,$users);
                }
                $this->mail_link = "customer_chat/".$ticket->externalid;
                $this->mail_link2 = config("app.cust_domain").$this->mail_link;

                dispatch(
                    function () use ($users,$action,$ticket)
                    {
                        $this->send_to_members("","","",$users,"false","2");
                        if($action == "escalation")
                        {
                            $this->send_to_customer( $this->mail_subject, $this->mail_body, $this->mail_link, $this->mail_link2,$ticket->company_id,"2");
                        }
                    }
                )->delay(now()->addSeconds(config("app.queue_time")));

                return redirect()->back()->with(['form_submit_flag' => 'true', 'form_submit_msg' =>"Note Added Successfully"]);
            }
            else
            {
                return redirect()->back()->with(['form_submit_flag' => 'false', 'form_submit_msg' =>"Note was not Added. Try Again"]);
            }
        }
        catch(Exception $e)
        {
            return redirect()->back()->with(['form_submit_flag' => 'false', 'form_submit_msg' =>"An exception Occured".$e->getMessage()]);
        }


        }
        return view("tickets.ticket_notes",compact("ticket","action"));
    }

    public function processing_tickets($type="")
    {
       $tickets = ticket::where("status","opened")->withCount(['get_chat'=> function($q)
       {
        $q->where("receiver","agent")->whereNull("agent_status");
       }])
       ->orderByRaw(DB::raw("(select ticket_chats.id from ticket_chats where ticket_chats.ticket_id=tickets.id order by ticket_chats.id desc limit 1) desc"))
       ->get();
    return view("tickets.processing_tickets", compact("type"));
    }





}
