<?php

use Illuminate\Database\Migrations\Migration;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Support\Facades\Schema;

class CreateTicketsTable extends Migration
{
    /**
     * Run the migrations.
     *
     * @return void
     */
    public function up()
    {
        Schema::create('tickets', function (Blueprint $table) {
            $table->id();
            $table->string("subject")->nullable();
            $table->string("externalid",400)->unique();
            $table->bigInteger("department_id")->unsigned()->nullable();
            $table->foreign("department_id")->references("id")->on("departments")->onDelete("cascade");
            $table->bigInteger("company_id")->unsigned()->nullable();
            $table->foreign("company_id")->references("id")->on("companies")->onDelete("cascade");
            $table->bigInteger("manager_id")->unsigned()->nullable();
            $table->foreign("manager_id")->references("id")->on("members")->onDelete("cascade");
            $table->string("manager_assigned_at")->dateTime();
            $table->bigInteger("supervisor_id")->unsigned()->nullable();
            $table->foreign("supervisor_id")->references("id")->on("members")->onDelete("cascade");
            $table->string("supervisor_assigned_at")->dateTime();
            $table->bigInteger("agent_id")->unsigned()->nullable();
            $table->foreign("agent_id")->references("id")->on("members")->onDelete("cascade");
            $table->string("agent_assigned_at")->dateTime();
            $table->string("internal_status")->nullable();
            $table->string("source")->nullable();//company,member
            $table->bigInteger("generated_by")->unsigned()->nullable();
            $table->foreign("generated_by")->references("id")->on("members")->onDelete("cascade");
            $table->bigInteger("closed_by")->unsigned()->nullable();
            $table->foreign("closed_by")->references("id")->on("members")->onDelete("cascade");
            $table->dateTime("closed_at")->nullable();

            $table->bigInteger("opened_by")->unsigned()->nullable();
            $table->foreign("opened_by")->references("id")->on("members")->onDelete("cascade");
            $table->dateTime("opened_at")->nullable();

            $table->integer("priority")->nullable();
            $table->float("time")->nullable();
            $table->string("time_status")->nullable();
            $table->string("status")->nullable();
            $table->timestamps();
        });
    }

    /**
     * Reverse the migrations.
     *
     * @return void
     */
    public function down()
    {
        Schema::dropIfExists('tickets');
    }
}
