@extends("layouts.adminmaster")


@section("title")
Add New Employee
@endsection

@section("content")

<div class="card">
    <div class="card-header">
        <div class="card-title">Add Employee</div>
    </div>
    <div class="card-body">
        <form action="" method="POST">
            @csrf
            <div class="form-group mb-2">
                <label class="text-muted">Name:</label>
                <input type="text" name="name" class="form-control form-control-sm"  value="{{ old('name') }}" placeholder="Enter Name...">
                <span class="text-danger">
                    @error('name')
                        {{ $message }}
                    @enderror
                </span>
            </div>
            <div class="form-group mb-2">
                <label class="text-muted">Email:</label>
                <input type="email" name="email" value="{{ old('email') }}" class="form-control form-control-sm" placeholder="Enter Email...">
                <span class="text-danger">
                    @error('email')
                        {{ $message }}
                    @enderror
                </span>
            </div>
            <div class="form-group mb-2">
                <label class="text-muted">Phone No:</label>
                <input type="number" name="phone" value="{{ old('phone') }}" class="form-control form-control-sm" placeholder="Enter Phone No...">
                <span class="text-danger">
                    @error('phone')
                        {{ $message }}
                    @enderror
                </span>
            </div>
            <div class="form-group mb-2">
                <label class="text-muted" id="">Choose Role:</label>
                <select name="role" id="choose_role" class="form-control">
                    <option value="">--Choose Role--</option>
                    @foreach ($roles as $role)
                    <option data-code="{{ $role->role_code }}" {{ (old('role') == $role->role_code) ? 'selected' : '' }} value="{{ $role->role_code }}">{{ $role->name }}</option>
                    @endforeach
                </select>
                <span class="text-danger">
                    @error('role')
                        {{ $message }}
                    @enderror
                </span>
            </div>


            <div class="form-group mb-2" id="man_div">
                <label class="text-muted">Choose Manager:</label>
                <select name="manager" id="" class="form-control ">
                    <option value="">--Choose Manager--</option>
                    @foreach ($managers as $sup)
                    <option {{ (old('manager') == $sup->id) ? 'selected' : '' }}  value="{{ $sup->id }}">{{ $sup->name }}</option>
                    @endforeach
                </select>
                <span class="text-danger">
                    @error('manager')
                        {{ $message }}
                    @enderror
                </span>
            </div>

            <div class="form-group mb-2" id="sup_div">
                <label class="text-muted">Choose Department:</label>
                <select name="department" id="" class="form-control ">
                    <option value="">--Choose Department--</option>
                    @foreach ($departments as $dep)
                    <option {{ (old('department') == $dep->id) ? 'selected' : '' }}  value="{{ $dep->id }}">{{ $dep->name }} ({{ $dep->get_supervisor->name ?? '' }})</option>
                    @endforeach
                </select>
                <span class="text-danger">
                    @error('department')
                        {{ $message }}
                    @enderror
                </span>
            </div>



            <div class="form-group mb-2">
                <label class="text-muted">Description:</label>
              <textarea name="description" class="form-control " placeholder="Enter Description (if any)...">{{ old('description') }}</textarea>
                <span class="text-danger">
                    @error('description')
                        {{ $message }}
                    @enderror
                </span>
            </div>
            <div class="text-right">
                <button class="btn btn-md btn-primary" type="submit">SUBMIT</button>
            </div>
        </form>
    </div>
</div>


@endsection
@section("scripts")
<script>
    var role = "{{ old('role') }}";
    if(role == "supervisor_13")
    {
        $("#sup_div").hide();

    }
    if(role == "agent_14")
    {
        $("#man_div").hide();

    }


    $(document).ready(function(){
        $('.add-emp').addClass('active');
        $('.emp').addClass('in');
        $("#choose_role").change(function(){
            var val = $(this).find(":selected").attr("data-code");


            if(val=="supervisor_13")
            {
                $("#sup_div").hide();
                $("#man_div").show();

            }
            else
            {
                $("#sup_div").show();
                $("#man_div").hide();

            }
        });
    });
</script>

@endsection

