@extends('layouts.adminmaster')


@section('content')

    <div class="card">
        <div class="card-header">
            <div class="card-title">
                {{ $role }} Tickets
            </div>
        </div>
        <div class="card-body">
            @if ($errors->any())
                @foreach ($errors->all() as $error)
                    <div class="text-danger size-nm">{{ $error }}</div>
                @endforeach
            @endif
            @if (session('cms_member_role') != 'super_admin_11')
                <div class="text-right">
                    <label for="" class="text-muted">Ticket Assigned Today:</label> {{ $tickets_count }}
                </div>
            @endif
            <form action="" class="row" method="GET">

                <div class="col-md-3">
                    <select name="department"  class="form-control  ">
                        <option value="">--Choose Department--</option>
                        @foreach ($departments as $dep)
                            <option value="{{ $dep->externalid }}" {{ $params == $dep->externalid ? 'selected' : '' }}>
                                {{ $dep->name }} ({{ $dep->get_supervisor->name ?? 'N/A' }})</option>
                        @endforeach
                    </select>
                    <span class="text-danger size-nm">
                        @error('department')
                            {{ $message }}
                        @enderror
                    </span>
                </div>
                <div class="col-md-2">
                    <input type="text"  name="ticket_id" value="{{ isset($filters['ticket_id']) ? $filters['ticket_id'] : '' }}" class="form-control form-control-sm" placeholder="Enter Ticket ID...">
                </div>
                <div class="col-md-2">
                    <select name="status"  class="form-control  ">
                        <option value="">--Choose Status--</option>
                        <option value="opened" {{ (($filters['status'] ?? '') == "opened") ? 'selected' : '' }}>Open</option>
                        <option value="closed" {{ (($filters['status'] ?? '') == "closed") ? 'selected' : '' }}>Closed</option>
                        <option value="pending" {{ (($filters['status'] ?? '') == "pending") ? 'selected' : '' }}>Pending</option>
                    </select>
                    <span class="text-danger size-nm">
                        @error('status')
                            {{ $message }}
                        @enderror
                    </span>
                </div>
                <div class="col-md-2">
                    <select name="assignment"  class="form-control  ">
                        <option value="">--Choose Assigned/Not Assigned--</option>
                        <option value="assigned" {{ (($filters['assignment'] ?? '') == "assigned") ? 'selected' : '' }}>Assigned</option>
                        <option value="not_assigned" {{ (($filters['assignment'] ?? '') == "not_assigned") ? 'selected' : '' }}>Not Assigned</option>
                    </select>
                    <span class="text-danger size-nm">
                        @error('assignment')
                            {{ $message }}
                        @enderror
                    </span>
                </div>
                <div class="col-md-1">
                    <input type="submit" value="Apply" class="btn btn-xs btn-primary">
                </div>
                <div class="col-md-2 text-right">
                    @if ($params != '' && session('cms_member_role') == 'supervisor_13')
                        <button type="button" id="agent-btn" class="btn btn-xs size-nm btn-primary" data-toggle='modal'
                            data-target='#agent_modal'>Assign Agents</button>
                    @endif

                </div>

                <div class="col-md-12 mt-4 text-right mb-2 ">
                    {{-- <button type="button" class="btn btn-rounded btn-info"><span class="btn-icon-left text-info"><i class="fa fa-plus color-info"></i>
                                    </span>Add</button> --}}
                    <a href="{{ route('generate_ticket') }}" class="btn btn-xs btn-rounded btn-info">
                    Create Ticket For Exiting Customer</a>

                    <a href="{{ route('create_ticket_customer') }}" class="btn btn-xs  btn-rounded p-1 btn-primary">Create Ticket For New Customer</a>
                </div>

            </form>
            <div class="table-responsive mb-3">
                <table class="table table-sm table-sm-responsive size-sm">

                        <tr>
                            <th>

                            </th>
                            <th>Company</th>
                            <th>Subject</th>

                            <th>Agent</th>
                            <th>ID</th>
                            <th>Status</th>
                            <th>Action</th>
                        </tr>


                        @foreach ($tickets as $t)
                            <tr>
                                {{-- <td>
                            <img src="{{ asset($t['get_company']['photo']) }}" class="img img-fluid logo-sm">
                        </td> --}}
                                <td>
                                    @if ($params != '')
                                        <label class="fancy-checkbox">
                                            <input class="checkbox-tick" type="checkbox" value="{{ $t->externalid }}"
                                                name="ticket_checkbox[]">
                                            <span></span>
                                    @endif
                                    </label>
                                </td>
                                <td>
                                    @if ($t->escalation == 'escalated')
                                        <p class="mb-0 text-right">
                                            <label style="font-size: 9px" for=""
                                                class="badge badge-danger">{{ $t->escalation }}</label>
                                        </p>
                                    @endif
                                    <img src="{{ $t->get_company->photo ?? '' }}" class="rounded-circle avatar logo-nm"
                                        alt="">
                                    <p class="c_name">{{ $t->get_company->name ?? '' }} <span
                                            class="badge badge-default m-l-10 hidden-sm-down">{{ $t->get_department->name ?? 'General' }}</span>
                                    </p>

                                </td>


                                <td class="project-title">
                                    <h6 class="size-nm font-weight-bold mb-0">{{ Str::limit($t->subject, 30) }}</h6>
                                    <small>Created at: {{ $t->created_at }}</small>

                                </td>
                                <td>

                                    <ul class="list-unstyled team-info">
                                        <label for=""
                                            class="size-sm mb-0 mt-1">{{ $t['get_agent']['name'] ?? '' }}</label>
                                        <br>
                                        <label for=""
                                            class="text-muted size-sm">{{ $t['get_agent']['email'] ?? '' }}</label>
                                        {{-- <br><li><img src="{{ asset($t->get_manager->photo ?? '') }}" alt="Avatar" title="{{ $t->get_manager->name ?? 'N/A' }}"></li>
                                <li><img src="{{ asset($t->get_supervisor->photo ?? '') }}" alt="Avatar" title="{{ $t->get_supervisor->name ?? 'N/A' }}"></li>
                                <li><img src="{{ asset($t->get_agent->photo ?? '') }}" alt="Avatar" title="{{ $t->get_agent->name ?? 'N/A' }}"></li> --}}
                                    </ul>
                                </td>
                                <td>
                                    <span for="" class="badge badge-primary">{{ $t->externalid }}</span>

                                </td>

                                <td>
                                    <x-ticket-status :ticket="$t"/>
                                </td>
                                <td class="project-actions">

                                    <button id="btnGroupDrop1" type="button"
                                        class="btn btn-sm size-nm btn-primary dropdown-toggle" data-toggle="dropdown"
                                        aria-haspopup="true" aria-expanded="false">
                                        Action
                                    </button>
                                    <div class="dropdown-menu" aria-labelledby="btnGroupDrop1" style="">
                                        <a href="{{ route('ticket_detail', $t->id) }}" class="dropdown-item size-nm"
                                            href="javascript:void(0);">View</a>
                                        <a href="{{ route('ticket_notes', $t->externalid) }}" class="dropdown-item size-nm"
                                            href="javascript:void(0);">Notes</a>
                                        <a href="{{ route('ticket_notes', [$t->externalid, 'escalation']) }}"
                                            class="size-nm dropdown-item" href="javascript:void(0);">Escalation</a>

                                        {{-- @if ($t->agent_id == session('cms_member_id')) --}}
                                        <a href="{{ route('ticket_chat', $t->externalid) }}"
                                            class="size-nm dropdown-item">Chat</a>
                                        {{-- @endif --}}
                                        @if ((session('cms_member_role') == 'manager_12' || session('cms_member_role') == 'supervisor_13') &&
                                            $t->status != 'closed')
                                            <button class="dropdown-item size-nm  pick_ticket_btn" data-toggle="modal"
                                                data-target="#pick_ticket_modal"
                                                data-ticket="{{ $t->externalid }}">Pick</button>
                                        @endif
                                    </div>
                                    {{-- <div class="btn-group">


                            <a href="{{ route('ticket_detail',$t->id) }}" class="size-sm btn btn-sm btn-primary">View</a>
                            <a href="{{ route('ticket_notes',$t->externalid) }}" class="size-sm btn btn-sm btn-primary">Notes</a>

                            @if ($t->agent_id == session('cms_member_id'))
                            <a href="{{ route('ticket_chat',$t->externalid) }}" class="size-sm btn btn-sm btn-warning text-dark">Chat</a>
                        </div>
                            @endif --}}
                                </td>
                            </tr>
                        @endforeach

                </table>
                <x-table-data :data="$tickets"/>


                <div class="text-right">
                {!! $tickets->links() !!}
            </div>
            </div>
            <div class="modal fade" id="pick_ticket_modal" tabindex="-1" role="dialog" aria-labelledby="exampleModalLabel"
                aria-hidden="true">
                <div class="modal-dialog" role="document">
                    <div class="modal-content">
                        <div class="modal-header">
                            <h5 class="modal-title" id="exampleModalLabel">Pick Up Ticket</h5>
                            <button type="button" class="close" data-dismiss="modal" aria-label="Close">
                                <span aria-hidden="true">&times;</span>
                            </button>
                        </div>
                        <div class="modal-body">

                            <form action="{{ route('pick_ticket') }}" method="POST">
                                @csrf
                                <input type="hidden" name="ticket_id" id="ticket_id_inp">
                                <h6>Are you really want to pick up the ticket <span id="ticket_id_span"
                                        class="font-weight-bold"></span> ? </h6>
                                <div class="text-right">
                                    <button type="button" class="btn btn-secondary" data-dismiss="modal">No</button>
                                    <button type="submit" class="btn btn-primary">Yes</button>
                                </div>
                            </form>
                        </div>
                    </div>
                </div>
            </div>
            @if ($params != '')
                <div class="modal fade" id="agent_modal">
                    <div class="modal-dialog modal-lg">
                        <div class="modal-content">
                            <div class="modal-header">
                                Assign Agent
                            </div>
                            <div class="modal-body">
                                <form method="POST" action="{{ route('submit_multiple_tickets') }}" id="agent_form">
                                    <h6 class="mb-1">Selected Tickets:</h6>
                                    <div id="agent_tickets" class="mb-2">

                                    </div>
                                    @csrf
                                    <label for="" class="form-label">Choose Agent</label><br>
                                    <select name="agent" class="form-control" style="width:50%">
                                        <option value="">--Choose Agent--</option>
                                        @foreach ($agents as $ag)
                                            <option value="{{ $ag->id }}">{{ $ag->name }}
                                                ({{ $ag->email }})</option>
                                        @endforeach
                                    </select>
                                    <div class="text-right mt-2">
                                        <button class="btn btn-sm btn-primary size-nm">Submit</button>
                                    </div>
                                </form>
                            </div>
                        </div>
                    </div>
                </div>
            @endif

        </div>
    </div>

@endsection

@section('scripts')
    <script>
        $(document).ready(function() {
             $('.view-ticket').addClass('active');
  $('.ticket').addClass('in');
            $("#agent-btn").click(function() {
                var a = "";
                $("#agent_tickets").html("");

                var checkboxes = document.querySelectorAll('input[type=checkbox]:checked');
                for (var i = 0; i < checkboxes.length; i++) {
                    a = "<input type='hidden' name='tickets[]' value='" + checkboxes[i].value +
                        "'><label class='badge badge-primary m-1'>" + checkboxes[i].value + "</label>";
                    $("#agent_tickets").append(a);
                }
            });

            $(".pick_ticket_btn").click(function(){
        $("#ticket_id_inp").val($(this).attr("data-ticket"));
        $("#ticket_id_span").text($(this).attr("data-ticket"));

        $("#pick_ticket_modal").modal("show");
    });
        });


    </script>
@endsection
