<?php

use App\Http\Controllers\ChatController;
use App\Http\Controllers\CompanyController;
use App\Http\Controllers\DepartmentController;
use Illuminate\Support\Facades\Route;
use App\Http\Controllers\MemberController;
use App\Http\Controllers\TicketController;
use App\Http\Controllers\TagsController;
use App\Http\Controllers\DashboardController;
use App\Http\Controllers\HomeController;
use App\Http\Controllers\NotificationController;
use App\Http\Controllers\CompaignController;
use App\Http\Controllers\EmailController;
use App\Http\Controllers\TemplateController;

use App\Http\Controllers\MailingListController;

/*
|--------------------------------------------------------------------------
| Web Routes
|--------------------------------------------------------------------------
|
| Here is where you can register web routes for your application. These
| routes are loaded by the RouteServiceProvider within a group which
| contains the "web" middleware group. Now create something great!
|
*/

Route::match(['get', 'post'], 'forget-password', [MemberController::class, "forget_password"])->name("forget-password");
Route::match(['get', 'post'], 'login', [MemberController::class, "login"])->name("login");

Route::group(['middleware' => ['user_auth']], function () {
    Route::get('get_dispatch_progress', [DashboardController::class, "get_dispatch_progress"])->name("get_dispatch_progress");
    Route::match(['get', 'post'], 'create_ticket_customer', [TicketController::class, "create_ticket_customer"])->name("create_ticket_customer");
    Route::get('my-profile', [MemberController::class, "my_profile"])->name("my-profile");
    Route::post('reset_my_password', [MemberController::class, "reset_my_password"])->name("reset_my_password");
    Route::get('member/{id?}', [MemberController::class, "member_profile"])->name("member");
    Route::get('dashboard', [DashboardController::class, "dashboard"])->name("dashboard");
    Route::match(['post', 'get'], 'edit_member/{id}', [MemberController::class, "edit_member"])->name("edit_member");
    Route::get('member_new_notifications', [NotificationController::class, "member_new_notifications"])->name("member_new_notifications");
    Route::get('mark_check_status', [NotificationController::class, "mark_check_status"])->name("mark_check_status");
    Route::get('employee_notifications', [NotificationController::class, "employee_notifications"])->name("employee_notifications");

    Route::match(['get', 'post'], 'import_customers', [CompanyController::class, "import_customers"])->name("import_customers");
    Route::match(['get', 'post'], 'add_member', [MemberController::class, "add_member"])->name("add_member");
    Route::match(['get', 'post'], 'add_manager', [MemberController::class, "add_manager"])->name("add_manager");
    Route::post('reset_password_by_manager', [MemberController::class, "reset_password_by_manager"])->name("reset_password_by_manager");
    Route::get('view_all_members', [MemberController::class, "view_all_members"])->name("view_all_members");
    Route::get('my_members', [MemberController::class, "my_members"])->name("my_members");
    Route::post('delete_department', [DepartmentController::class, "delete_department"])->name("delete_department");

    Route::get('view_departments', [DepartmentController::class, "view_departments"])->name("view_departments");
    Route::match(['get', 'post'], 'add_department', [DepartmentController::class, "add_department"])->name("add_department");
    Route::get('department_profile/{id}', [DepartmentController::class, "department_profile"])->name("department_profile");
    Route::get('my_departments', [DepartmentController::class, "my_departments"])->name("my_departments");
    Route::match(['get', 'post'], 'edit_department/{id}', [DepartmentController::class, "edit_department"])->name("edit_department");
    Route::get('view_tickets', [TicketController::class, "view_tickets"])->name("view_tickets");
    Route::match(['get', 'post'], 'generate_ticket', [TicketController::class, "generate_ticket"])->name("generate_ticket");
    Route::get('processing_tickets/{type?}', [TicketController::class, "processing_tickets"])->name("processing_tickets");

    Route::post('submit_multiple_tickets', [TicketController::class, "submit_multiple_tickets"])->name("submit_multiple_tickets");
    Route::get('ticket_detail/{id}', [TicketController::class, "ticket_detail"])->name("ticket_detail");
    Route::post('submit_ticket_detail', [TicketController::class, "submit_ticket_detail"])->name("submit_ticket_detail");
    Route::match(['get', 'post'], 'ticket_chat/{id}', [TicketController::class, "ticket_chat"])->name("ticket_chat");
    Route::get('my_departments_tickets', [TicketController::class, "my_department_tickets"])->name("my_departments_tickets");
    Route::post('pick_ticket', [TicketController::class, "pick_ticket"])->name("pick_ticket");
    Route::get('company_profile/{id}', [CompanyController::class, "company_profile"])->name("company_profile");
    Route::post('add_tag', [TagsController::class, "add_tag"])->name("add_tag");
    Route::get('view_tags', [TagsController::class, "view_tags"])->name("view_tags");
    Route::match(['get', 'post'], 'add_company', [CompanyController::class, "add_company"])->name("add_company");
    Route::get('view_companies', [CompanyController::class, "view_companies"])->name("view_companies");
    Route::get('all_tickets', [TicketController::class, "all_tickets"])->name("all_tickets");
    Route::get('get_agent_chat/{id}/{type}', [ChatController::class, "get_agent_chat"])->name("get_agent_chat");
    Route::post('close_ticket', [TicketController::class, "close_ticket"])->name("close_ticket");

    Route::match(['post', 'get'], 'ticket_notes/{id}/{action?}', [TicketController::class, "ticket_notes"])->name("ticket_notes");

    Route::match(['get','post'], 'logout', [MemberController::class, "logout"])->name("logout");
    Route::post('submit_agent_chat', [ChatController::class, "submit_agent_chat"])->name("submit_agent_chat");
    Route::match(['post', 'get'], 'edit_company/{id}', [CompanyController::class, "edit_company"])->name("edit_company");
    Route::post('edit_tags', [CompanyController::class, "edit_tags"])->name("edit_tags");

    ///////////////////////////////////////////////////////////COMPAIGN///////////////////////////////////
    Route::prefix('compaign')->group(function () {
        Route::get('public_profile/{id}', [CompaignController::class, "public_profile"])->name("compaign.public_profile");

        Route::match(['post', 'get'], 'create', [CompaignController::class, "create"])->name("compaign.create");
        Route::get('view_all', [CompaignController::class, "view_all"])->name("compaign.view_all");
        Route::match(['post', 'get'], 'editor/{id}', [CompaignController::class, "editor"])->name("compaign.editor");
        Route::post('update_content', [CompaignController::class, "update_content"])->name("compaign.update_content");
        Route::get('profile/{id}', [CompaignController::class, "profile"])->name("compaign.profile");
        Route::get('recipients/{id}/{type?}', [CompaignController::class, "recipients"])->name("compaign.recipients");
        Route::post('save_recipients', [CompaignController::class, "save_recipients"])->name("compaign.save_recipients");
        Route::match(['post', 'get'], 'share/{id}', [CompaignController::class, "share"])->name("compaign.share");
        Route::get('recipient_sharing_detail/{pid}', [CompaignController::class, "recipient_sharing_detail"])->name("compaign.recipient_sharing_detail");
        Route::match(['post', 'get'], 'edit/{id}', [CompaignController::class, "edit"])->name("compaign.edit");
        Route::post('delete', [CompaignController::class, "delete"])->name("compaign.delete");

        Route::prefix('mailing_list')->group(
            function () {
                Route::get('import', [MailingListController::class, "import"])->name("compaign.mailing_list.import");
                Route::post('submit_import', [MailingListController::class, "submit_import"])->name("compaign.mailing_list.submit_import");
                Route::post('add_file_name', [MailingListController::class, "add_file_name"])->name("compaign.mailing_list.add_file_name");
                Route::get('view', [MailingListController::class, "view"])->name("compaign.mailing_list.view");
                Route::match(['post', 'get'], 'create', [MailingListController::class, "create"])->name("compaign.mailing_list.create");
                Route::match(['post', 'get'], 'edit/{id}', [MailingListController::class, "edit"])->name("compaign.mailing_list.edit");
            }
        );
        Route::prefix('templates')->group(function () {
            Route::match(['get', 'post'], 'edit/{id}', [TemplateController::class, "edit"])->name("compaign.templates.edit");
            Route::match(['get', 'post'], 'view', [TemplateController::class, "view"])->name("compaign.templates.view");
            Route::match(['get', 'post'], 'create/{id}', [TemplateController::class, "create"])->name("compaign.templates.create");
            Route::post('update_content', [TemplateController::class, "update_content"])->name("compaign.templates.update_content");
            Route::post('delete', [TemplateController::class, "delete"])->name("compaign.templates.delete");
        });
    });
    Route::prefix('email')->group(function () {
        Route::match(['post', 'get'], 'emails', [EmailController::class, "emails"])->name("email.emails");
        Route::match(['post', 'get'], 'edit/{id}', [EmailController::class, "edit"])->name("email.edit");
    });
});


Route::group(['middleware' => ['company_auth']], function () {
    Route::post('reset_password', [CompanyController::class, "reset_password"])->name("reset_password");
    Route::post('edit_profile', [CompanyController::class, "edit_profile"])->name("edit_profile");
    Route::get('my_tickets', [TicketController::class, "my_tickets"])->name("my_tickets");
    Route::get('my_profile', [CompanyController::class, "my_profile"])->name("my_profile");
    Route::get('customer_chat/{id}', [ChatController::class, "customer_chat"])->name("customer_chat");
    Route::get('get_customer_chat/{id}/{type}', [ChatController::class, "get_customer_chat"])->name("get_customer_chat");
    Route::post('submit_customer_chat', [ChatController::class, "submit_customer_chat"])->name("submit_customer_chat");

    Route::get('customer_new_notifications', [NotificationController::class, "customer_new_notifications"])->name("customer_new_notifications");
    Route::get('customer_mark_check_status', [NotificationController::class, "customer_mark_check_status"])->name("customer_mark_check_status");
    Route::get('customer_notifications', [NotificationController::class, "customer_notifications"])->name("customer_notifications");
});

Route::match(['get', 'post'], 'forget_password', [CompanyController::class, "forget_password"])->name("forget_password");
Route::match(['get', 'post'], 'register', [CompanyController::class, "register"])->name("register");
Route::match(['get', 'post'], 'company-login', [CompanyController::class, "company_login"])->name("company-login");
Route::match(['get','post'],'company_logout', [CompanyController::class, "company_logout"])->name("company_logout");
Route::match(['get', 'post'], 'create_ticket', [TicketController::class, "create_ticket"])->name("create_ticket");
Route::get('/home', [HomeController::class, "home"])->name("home");
Route::match(['get', 'post'], 'verify_otp', [CompanyController::class, "verify_otp"])->name("verify_otp");
Route::match(['get', 'post'], 'set_password', [CompanyController::class, "set_password"])->name("set_password");
